# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'edit_event.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QGroupBox,
    QHBoxLayout, QLabel, QPlainTextEdit, QPushButton,
    QSizePolicy, QSpacerItem, QSpinBox, QVBoxLayout,
    QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(600, 638)
        self.verticalLayout_3 = QVBoxLayout(Form)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.gb_time = QGroupBox(Form)
        self.gb_time.setObjectName(u"gb_time")
        self.verticalLayout = QVBoxLayout(self.gb_time)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.cb_set_time_na = QCheckBox(self.gb_time)
        self.cb_set_time_na.setObjectName(u"cb_set_time_na")

        self.horizontalLayout_8.addWidget(self.cb_set_time_na)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_8.addItem(self.horizontalSpacer_7)


        self.verticalLayout.addLayout(self.horizontalLayout_8)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.lb_time = QLabel(self.gb_time)
        self.lb_time.setObjectName(u"lb_time")

        self.horizontalLayout_3.addWidget(self.lb_time)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)


        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)


        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.pb_set_to_current_time = QPushButton(self.gb_time)
        self.pb_set_to_current_time.setObjectName(u"pb_set_to_current_time")

        self.horizontalLayout_9.addWidget(self.pb_set_to_current_time)

        self.horizontalSpacer_8 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_9.addItem(self.horizontalSpacer_8)


        self.verticalLayout.addLayout(self.horizontalLayout_9)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)


        self.verticalLayout_3.addWidget(self.gb_time)

        self.gb_image_index = QGroupBox(Form)
        self.gb_image_index.setObjectName(u"gb_image_index")
        self.verticalLayout_2 = QVBoxLayout(self.gb_image_index)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.sb_image_idx = QSpinBox(self.gb_image_index)
        self.sb_image_idx.setObjectName(u"sb_image_idx")
        self.sb_image_idx.setMaximum(10000000)

        self.horizontalLayout_7.addWidget(self.sb_image_idx)

        self.pb_set_to_current_image_index = QPushButton(self.gb_image_index)
        self.pb_set_to_current_image_index.setObjectName(u"pb_set_to_current_image_index")

        self.horizontalLayout_7.addWidget(self.pb_set_to_current_image_index)

        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer_6)


        self.verticalLayout_2.addLayout(self.horizontalLayout_7)


        self.verticalLayout_3.addWidget(self.gb_image_index)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.lbSubject = QLabel(Form)
        self.lbSubject.setObjectName(u"lbSubject")

        self.horizontalLayout_4.addWidget(self.lbSubject)

        self.cobSubject = QComboBox(Form)
        self.cobSubject.setObjectName(u"cobSubject")

        self.horizontalLayout_4.addWidget(self.cobSubject)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer_2)


        self.verticalLayout_3.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_2 = QLabel(Form)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout_5.addWidget(self.label_2)

        self.cobCode = QComboBox(Form)
        self.cobCode.setObjectName(u"cobCode")

        self.horizontalLayout_5.addWidget(self.cobCode)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_5.addItem(self.horizontalSpacer_3)


        self.verticalLayout_3.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.lb_frame_idx = QLabel(Form)
        self.lb_frame_idx.setObjectName(u"lb_frame_idx")

        self.horizontalLayout_6.addWidget(self.lb_frame_idx)

        self.sb_frame_idx = QSpinBox(Form)
        self.sb_frame_idx.setObjectName(u"sb_frame_idx")
        self.sb_frame_idx.setMinimum(0)
        self.sb_frame_idx.setMaximum(100000000)

        self.horizontalLayout_6.addWidget(self.sb_frame_idx)

        self.cb_set_frame_idx_na = QCheckBox(Form)
        self.cb_set_frame_idx_na.setObjectName(u"cb_set_frame_idx_na")

        self.horizontalLayout_6.addWidget(self.cb_set_frame_idx_na)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_6.addItem(self.horizontalSpacer_5)


        self.verticalLayout_3.addLayout(self.horizontalLayout_6)

        self.label_4 = QLabel(Form)
        self.label_4.setObjectName(u"label_4")

        self.verticalLayout_3.addWidget(self.label_4)

        self.leComment = QPlainTextEdit(Form)
        self.leComment.setObjectName(u"leComment")

        self.verticalLayout_3.addWidget(self.leComment)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer_4 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_4)

        self.pbCancel = QPushButton(Form)
        self.pbCancel.setObjectName(u"pbCancel")

        self.horizontalLayout.addWidget(self.pbCancel)

        self.pbOK = QPushButton(Form)
        self.pbOK.setObjectName(u"pbOK")

        self.horizontalLayout.addWidget(self.pbOK)


        self.verticalLayout_3.addLayout(self.horizontalLayout)


        self.retranslateUi(Form)

        self.pbOK.setDefault(True)


        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Edit event", None))
        self.gb_time.setTitle(QCoreApplication.translate("Form", u"Time", None))
        self.cb_set_time_na.setText(QCoreApplication.translate("Form", u"Set time to NA", None))
        self.lb_time.setText("")
        self.pb_set_to_current_time.setText(QCoreApplication.translate("Form", u"Set to current time", None))
        self.gb_image_index.setTitle(QCoreApplication.translate("Form", u"Image index", None))
        self.pb_set_to_current_image_index.setText(QCoreApplication.translate("Form", u"Set to current image index", None))
        self.lbSubject.setText(QCoreApplication.translate("Form", u"Subject", None))
        self.label_2.setText(QCoreApplication.translate("Form", u"Behavior", None))
        self.lb_frame_idx.setText(QCoreApplication.translate("Form", u"Frame index", None))
        self.cb_set_frame_idx_na.setText(QCoreApplication.translate("Form", u"Set NA", None))
        self.label_4.setText(QCoreApplication.translate("Form", u"Comment", None))
        self.pbCancel.setText(QCoreApplication.translate("Form", u"Cancel", None))
        self.pbOK.setText(QCoreApplication.translate("Form", u"OK", None))
    # retranslateUi




"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2025 Olivier Friard

This file is part of BORIS.

  BORIS is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  any later version.

  BORIS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not see <http://www.gnu.org/licenses/>.

"""

import datetime as dt
import logging
import math
import os
import tablib
import pathlib as pl
from decimal import Decimal as dec

from . import observation_operations
from . import utilities as util
from . import config as cfg
from . import select_observations
from . import export_observation
from . import select_subj_behav
from . import project_functions
from . import dialog
from . import db_functions

from PySide6.QtWidgets import QApplication, QFileDialog, QMessageBox, QInputDialog


def export_events_as_behavioral_sequences(self, separated_subjects=False, timed=False):
    """
    export events from selected observations by subject as behavioral sequences (plain text file)
    behaviors are separated by character specified in self.behav_seq_separator (usually pipe |)
    for use with Behatrix (see https://www.boris.unito.it/pages/behatrix)

    Args:
        separated_subjects (bool):
        timed (bool):
    """

    # ask user for observations to analyze
    _, selected_observations = select_observations.select_observations2(
        self, cfg.MULTIPLE, "Select observations to export as behavioral sequences"
    )

    if not selected_observations:
        return

    # check if coded behaviors are defined in ethogram
    if project_functions.check_coded_behaviors_in_obs_list(self.pj, selected_observations):
        return

    # check if state events are paired
    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
    if not_ok or not selected_observations:
        return

    if len(selected_observations) == 1:
        max_media_duration_all_obs, _ = observation_operations.media_duration(self.pj[cfg.OBSERVATIONS], selected_observations)
        start_coding, end_coding, _ = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], selected_observations)
        start_interval, end_interval = observation_operations.time_intervals_range(self.pj[cfg.OBSERVATIONS], selected_observations)
    else:
        max_media_duration_all_obs = None
        start_coding, end_coding = dec("NaN"), dec("NaN")
        start_interval, end_interval = None, None

    parameters = select_subj_behav.choose_obs_subj_behav_category(
        self,
        selected_observations,
        start_coding=start_coding,
        end_coding=end_coding,
        start_interval=start_interval,
        end_interval=end_interval,
        maxTime=max_media_duration_all_obs,
        show_include_modifiers=True,
        show_exclude_non_coded_behaviors=False,
        n_observations=len(selected_observations),
    )

    if parameters == {}:
        return
    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
        return

    file_name, _ = QFileDialog.getSaveFileName(self, "Export events as behavioral sequences", "", "Text files (*.txt);;All files (*)")

    if not file_name:
        return
    r, msg = export_observation.observation_to_behavioral_sequences(
        pj=self.pj,
        selected_observations=selected_observations,
        parameters=parameters,
        behaviors_separator=self.behav_seq_separator,
        separated_subjects=separated_subjects,
        timed=timed,
        file_name=file_name,
    )
    if not r:
        logging.critical(f"Error while exporting events as behavioral sequences: {msg}")
        QMessageBox.critical(
            None,
            cfg.programName,
            f"Error while exporting events as behavioral sequences:<br>{msg}",
            QMessageBox.Ok | QMessageBox.Default,
            QMessageBox.NoButton,
        )


def export_tabular_events(self, mode: str = "tabular") -> None:
    """
    * select observations
    * export events from the selected observations in various formats: TSV, CSV, ODS, XLSX, XLS, HTML

    Args:
        mode (str): export mode: must be ["tabular", "jwatcher"]
    """

    # ask user observations to analyze
    _, selected_observations = select_observations.select_observations2(
        self, cfg.MULTIPLE, windows_title="Select observations for exporting events"
    )

    if not selected_observations:
        return

    if mode == "jwatcher":
        # check if images observation in list
        max_obs_length, _ = observation_operations.observation_length(self.pj, selected_observations)

        # exit with message if events do not have timestamp
        if max_obs_length.is_nan():
            QMessageBox.critical(
                None,
                cfg.programName,
                ("This function is not available for observations with events that do not have timestamp"),
                QMessageBox.Ok | QMessageBox.Default,
                QMessageBox.NoButton,
            )
            return

    # check if coded behaviors are defined in ethogram
    if project_functions.check_coded_behaviors_in_obs_list(self.pj, selected_observations):
        return

    # check if state events are paired
    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
    if not_ok or not selected_observations:
        return

    if len(selected_observations) == 1:
        max_media_duration_all_obs, _ = observation_operations.media_duration(self.pj[cfg.OBSERVATIONS], selected_observations)
        start_coding, end_coding, _ = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], selected_observations)
        start_interval, end_interval = observation_operations.time_intervals_range(self.pj[cfg.OBSERVATIONS], selected_observations)
    else:
        max_media_duration_all_obs = None
        start_coding, end_coding = dec("NaN"), dec("NaN")
        start_interval, end_interval = None, None

    parameters = select_subj_behav.choose_obs_subj_behav_category(
        self,
        selected_observations,
        start_coding=start_coding,
        end_coding=end_coding,
        start_interval=start_interval,
        end_interval=end_interval,
        maxTime=max_media_duration_all_obs,
        show_include_modifiers=False,
        show_exclude_non_coded_behaviors=False,
        n_observations=len(selected_observations),
    )
    if parameters == {}:
        return
    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
        return

    if mode == "tabular":
        available_formats = (
            cfg.TSV,
            cfg.CSV,
            cfg.ODS,
            cfg.XLSX,
            cfg.XLS,
            cfg.HTML,
            cfg.PANDAS_DF,
            cfg.RDS,
        )
        if len(selected_observations) > 1:  # choose directory for exporting observations
            item, ok = QInputDialog.getItem(
                self,
                "Export events format",
                "Available formats",
                available_formats,
                0,
                False,
            )
            if not ok:
                return
            output_format = cfg.FILE_NAME_SUFFIX[item]

            exportDir = QFileDialog().getExistingDirectory(
                self,
                "Choose a directory to export events",
                os.path.expanduser("~"),
                options=QFileDialog.ShowDirsOnly,
            )
            if not exportDir:
                return

        if len(selected_observations) == 1:
            file_dialog_options = QFileDialog.Options()
            file_dialog_options |= QFileDialog.DontConfirmOverwrite

            file_name, filter_ = QFileDialog().getSaveFileName(
                self, "Export events", "", ";;".join(available_formats), options=file_dialog_options
            )
            if not file_name:
                return

            output_format = cfg.FILE_NAME_SUFFIX[filter_]
            if pl.Path(file_name).suffix != "." + output_format:
                file_name = str(pl.Path(file_name)) + "." + output_format
                # check if file with new extension already exists
                if pl.Path(file_name).exists():
                    if (
                        dialog.MessageDialog(cfg.programName, f"The file {file_name} already exists.", [cfg.CANCEL, cfg.OVERWRITE])
                        == cfg.CANCEL
                    ):
                        return

    if mode == "jwatcher":
        exportDir = QFileDialog().getExistingDirectory(
            self, "Choose a directory to export events", os.path.expanduser("~"), options=QFileDialog.ShowDirsOnly
        )
        if not exportDir:
            return

        output_format = "dat"

    mem_command = ""  # remember user choice when file already exists
    for obs_id in selected_observations:
        if len(selected_observations) > 1 or mode == "jwatcher":
            file_name = f"{pl.Path(exportDir) / util.safeFileName(obs_id)}.{output_format}"
            # check if file with new extension already exists
            if mem_command != cfg.OVERWRITE_ALL and pl.Path(file_name).is_file():
                if mem_command == cfg.SKIP_ALL:
                    continue
                mem_command = dialog.MessageDialog(
                    cfg.programName,
                    f"The file {file_name} already exists.",
                    [cfg.OVERWRITE, cfg.OVERWRITE_ALL, cfg.SKIP, cfg.SKIP_ALL, cfg.CANCEL],
                )
                if mem_command == cfg.CANCEL:
                    return
                if mem_command in [cfg.SKIP, cfg.SKIP_ALL]:
                    continue

        if mode == "tabular":
            r, msg = export_observation.export_tabular_events(
                self.pj,
                parameters,
                obs_id,
                self.pj[cfg.OBSERVATIONS][obs_id],
                self.pj[cfg.ETHOGRAM],
                file_name,
                output_format,
            )

        if mode == "jwatcher":
            r, msg = export_observation.export_events_jwatcher(
                parameters, obs_id, self.pj[cfg.OBSERVATIONS][obs_id], self.pj[cfg.ETHOGRAM], file_name, output_format
            )

        if not r and msg:
            QMessageBox.critical(None, cfg.programName, msg, QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)


def export_aggregated_events(self):
    """
    - select observations.
    - select subjects and behaviors
    - export events in aggregated format

    Formats can be SQL (sql), SDIS (sds), Tabular format (tsv, csv, ods, xlsx, xls, html) or Pandas dataframe
    """

    def fields_type(max_modif_number: int) -> dict:
        fields_type_dict: dict = {
            "Observation id": str,
            "Observation date": dt.datetime,
            "Description": str,
            "Observation type": str,
            "Source": str,
            "Time offset (s)": str,
            "Coding duration": float,
            "Media duration (s)": str,
            "FPS (frame/s)": str,
        }
        # TODO: "Media duration (s)" and "FPS (frame/s)" can be float for observation from 1 video

        if cfg.INDEPENDENT_VARIABLES in self.pj:
            for idx in util.sorted_keys(self.pj[cfg.INDEPENDENT_VARIABLES]):
                if self.pj[cfg.INDEPENDENT_VARIABLES][idx]["type"] == "timestamp":
                    fields_type_dict[self.pj[cfg.INDEPENDENT_VARIABLES][idx]["label"]] = dt.datetime
                elif self.pj[cfg.INDEPENDENT_VARIABLES][idx]["type"] == "numeric":
                    fields_type_dict[self.pj[cfg.INDEPENDENT_VARIABLES][idx]["label"]] = float
                else:
                    fields_type_dict[self.pj[cfg.INDEPENDENT_VARIABLES][idx]["label"]] = str

        fields_type_dict.update(
            {
                "Subject": str,
                "Observation duration by subject by observation": float,
                "Behavior": str,
                "Behavioral category": str,
            }
        )

        # max number of modifiers
        for i in range(max_modif_number):
            fields_type_dict[f"Modifier #{i + 1}"] = str

        fields_type_dict.update(
            {
                "Behavior type": str,
                "Start (s)": float,
                "Stop (s)": float,
                "Duration (s)": float,
                "Media file name": str,
                "Image index start": float,  # add image index and image file path to header
                "Image index stop": float,
                "Image file path start": str,
                "Image file path stop": str,
                "Comment start": str,
                "Comment stop": str,
            }
        )

        return fields_type_dict

    _, selected_observations = select_observations.select_observations2(self, cfg.MULTIPLE, "Select observations for exporting events")
    if not selected_observations:
        return

    # check if coded behaviors are defined in ethogram
    if project_functions.check_coded_behaviors_in_obs_list(self.pj, selected_observations):
        return

    # check if state events are paired
    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
    if not_ok or not selected_observations:
        return

    if len(selected_observations) == 1:
        max_media_duration_all_obs, _ = observation_operations.media_duration(self.pj[cfg.OBSERVATIONS], selected_observations)
        start_coding, end_coding, _ = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], selected_observations)
        start_interval, end_interval = observation_operations.time_intervals_range(self.pj[cfg.OBSERVATIONS], selected_observations)
    else:
        max_media_duration_all_obs = None
        start_coding, end_coding = dec("NaN"), dec("NaN")
        start_interval, end_interval = None, None

    parameters = select_subj_behav.choose_obs_subj_behav_category(
        self,
        selected_observations,
        start_coding=start_coding,
        end_coding=end_coding,
        start_interval=start_interval,
        end_interval=end_interval,
        maxTime=max_media_duration_all_obs,
        show_include_modifiers=False,
        show_exclude_non_coded_behaviors=False,
        n_observations=len(selected_observations),
    )
    if parameters == {}:
        return
    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to export")
        return

    # check for grouping results
    flag_group = True
    if len(selected_observations) > 1:
        flag_group = (
            dialog.MessageDialog(cfg.programName, "Group events from selected observations in one file?", [cfg.YES, cfg.NO]) == cfg.YES
        )

    if flag_group:
        file_formats = (
            cfg.TSV,
            cfg.CSV,
            cfg.ODS,
            cfg.XLSX,
            cfg.XLS,
            cfg.HTML,
            cfg.SDIS,
            cfg.TBS,
            cfg.SQL,
            cfg.PANDAS_DF,
            cfg.RDS,
        )

        file_dialog_options = QFileDialog.Options()
        file_dialog_options |= QFileDialog.DontConfirmOverwrite

        fileName, filter_ = QFileDialog().getSaveFileName(
            self, "Export aggregated events", "", ";;".join(file_formats), options=file_dialog_options
        )

        if not fileName:
            return

        outputFormat = cfg.FILE_NAME_SUFFIX[filter_]
        if pl.Path(fileName).suffix != "." + outputFormat:
            # check if file with new extension already exists
            fileName = str(pl.Path(fileName)) + "." + outputFormat
            if pl.Path(fileName).exists():
                if dialog.MessageDialog(cfg.programName, f"The file {fileName} already exists.", [cfg.CANCEL, cfg.OVERWRITE]) == cfg.CANCEL:
                    return

    else:  # not grouping
        file_formats = (
            cfg.TSV,
            cfg.CSV,
            cfg.ODS,
            cfg.XLSX,
            cfg.XLS,
            cfg.HTML,
            cfg.SDIS,
            cfg.TBS,
            cfg.PANDAS_DF,
            cfg.RDS,
        )
        item, ok = QInputDialog.getItem(self, "Export events format", "Available formats", file_formats, 0, False)
        if not ok:
            return
        # read the output format code
        outputFormat = cfg.FILE_NAME_SUFFIX[item]

        exportDir = QFileDialog().getExistingDirectory(
            self, "Choose a directory to export events", os.path.expanduser("~"), options=QFileDialog.ShowDirsOnly
        )
        if not exportDir:
            return

    if outputFormat == cfg.SQL_EXT:
        _, _, conn = db_functions.load_aggregated_events_in_db(
            self.pj, parameters[cfg.SELECTED_SUBJECTS], selected_observations, parameters[cfg.SELECTED_BEHAVIORS]
        )
        try:
            with open(fileName, "w") as f:
                for line in conn.iterdump():
                    f.write(f"{line}\n")
        except Exception:
            QMessageBox.critical(
                None,
                cfg.programName,
                f"The file {fileName} can not be saved",
                QMessageBox.Ok | QMessageBox.Default,
                QMessageBox.NoButton,
            )

        return

    # compute the maximum number of modifiers
    tot_max_modifiers: int = 0
    for obs_id in selected_observations:
        _, max_modifiers = export_observation.export_aggregated_events(self.pj, parameters, obs_id)
        tot_max_modifiers = max(tot_max_modifiers, max_modifiers)

    logging.debug(f"tot_max_modifiers: {tot_max_modifiers}")

    data_grouped_obs = tablib.Dataset()

    mem_command: str = ""  # remember user choice when file already exists
    header = list(fields_type(tot_max_modifiers).keys())

    for obs_id in selected_observations:
        logging.debug(f"Exporting aggregated events for obs Id: {obs_id}")

        data_single_obs, _ = export_observation.export_aggregated_events(
            self.pj, parameters, obs_id, force_number_modifiers=tot_max_modifiers
        )

        try:
            # order by start time
            index = header.index("Start (s)")
            if cfg.NA not in [x[index] for x in list(data_single_obs)]:
                data_single_obs_sorted = tablib.Dataset(
                    *sorted(list(data_single_obs), key=lambda x: float(x[index])),
                    headers=list(fields_type(tot_max_modifiers).keys()),
                )
            else:
                # order by image index
                index = header.index("Image index start")
                data_single_obs_sorted = tablib.Dataset(
                    *sorted(list(data_single_obs), key=lambda x: float(x[index])),
                    headers=list(fields_type(tot_max_modifiers).keys()),
                )
        except Exception:
            # if error no order
            data_single_obs_sorted = tablib.Dataset(
                *list(data_single_obs),
                headers=list(fields_type(tot_max_modifiers).keys()),
            )

        data_single_obs_sorted.title = obs_id

        if (not flag_group) and (outputFormat not in (cfg.SDIS_EXT, cfg.TBS_EXT)):
            fileName = f"{pl.Path(exportDir) / util.safeFileName(obs_id)}.{outputFormat}"
            # check if file with new extension already exists
            if mem_command != cfg.OVERWRITE_ALL and pl.Path(fileName).is_file():
                if mem_command == cfg.SKIP_ALL:
                    continue
                mem_command = dialog.MessageDialog(
                    cfg.programName,
                    f"The file {fileName} already exists.",
                    [cfg.OVERWRITE, cfg.OVERWRITE_ALL, cfg.SKIP, cfg.SKIP_ALL, cfg.CANCEL],
                )
                if mem_command == cfg.CANCEL:
                    return
                if mem_command in (cfg.SKIP, cfg.SKIP_ALL):
                    continue

            r, msg = export_observation.dataset_write(data_single_obs_sorted, fileName, outputFormat, dtype=fields_type(max_modifiers))
            if not r:
                QMessageBox.warning(None, cfg.programName, msg, QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)

        """
        # disabled after introduction of the force_number_modifiers parameter in export_aggregated_events function
        if len(data_single_obs_sorted) and max_modifiers < tot_max_modifiers:
            for i in range(tot_max_modifiers - max_modifiers):
                data_single_obs_sorted.insert_col(
                    14,
                    col=[""] * (len(list(data_single_obs_sorted))),
                    header=f"Modif #{i}",
                )
        """

        data_grouped_obs.extend(data_single_obs_sorted)

    data_grouped_obs_all = tablib.Dataset(headers=list(fields_type(tot_max_modifiers).keys()))

    data_grouped_obs_all.extend(data_grouped_obs)
    data_grouped_obs_all.title = "Aggregated events"

    start_idx = header.index("Start (s)")
    stop_idx = header.index("Stop (s)")

    if outputFormat == cfg.TBS_EXT:  # Timed behavioral sequences
        out: str = ""
        for obs_id in selected_observations:
            # observation id
            out += f"# {obs_id}\n"

            for event in list(data_grouped_obs_all):
                if event[0] == obs_id:
                    behavior = event[header.index("Behavior")]
                    subject = event[header.index("Subject")]
                    # replace various char by _
                    for char in (" ", "-", "/"):
                        behavior = behavior.replace(char, "_")
                        subject = subject.replace(char, "_")
                    event_start = f"{float(event[start_idx]):.3f}"  # start event
                    if not event[stop_idx]:  # stop event (from end)
                        event_stop = f"{float(event[start_idx]) + 0.001:.3f}"
                    else:
                        event_stop = f"{float(event[stop_idx]):.3f}"

                    bs_timed = [f"{subject}_{behavior}"] * round((float(event_stop) - float(event_start)) * 100)
                    out += "|".join(bs_timed)

            out += "\n"

            if not flag_group:
                fileName = f"{pl.Path(exportDir) / util.safeFileName(obs_id)}.{outputFormat}"
                with open(fileName, "wb") as f:
                    f.write(str.encode(out))
                out = ""

        if flag_group:
            with open(fileName, "wb") as f:
                f.write(str.encode(out))
        return

    if outputFormat == cfg.SDIS_EXT:  # SDIS format
        out: str = f"% SDIS file created by BORIS (www.boris.unito.it) at {util.datetime_iso8601(dt.datetime.now())}\nTimed <seconds>;\n"
        for obs_id in selected_observations:
            # observation id
            out += "\n<{}>\n".format(obs_id)

            for event in list(data_grouped_obs_all):
                if event[0] == obs_id:
                    behavior = event[header.index("Behavior")]
                    subject = event[header.index("Subject")]
                    # replace various char by _
                    for char in (" ", "-", "/"):
                        behavior = behavior.replace(char, "_")
                        subject = subject.replace(char, "_")

                    event_start = f"{float(event[start_idx]):.3f}"  # start event
                    if not event[stop_idx]:  # stop event (from end)
                        event_stop = f"{float(event[start_idx]) + 0.001:.3f}"
                    else:
                        event_stop = f"{float(event[stop_idx]):.3f}"
                    out += f"{subject}_{behavior},{event_start}-{event_stop} "

            out += "/\n\n"
            if not flag_group:
                fileName = f"{pl.Path(exportDir) / util.safeFileName(obs_id)}.{outputFormat}"
                with open(fileName, "wb") as f:
                    f.write(str.encode(out))
                out = f"% SDIS file created by BORIS (www.boris.unito.it) at {util.datetime_iso8601(dt.datetime.now())}\nTimed <seconds>;\n"

        if flag_group:
            with open(fileName, "wb") as f:
                f.write(str.encode(out))
        return

    if flag_group:
        r, msg = export_observation.dataset_write(data_grouped_obs_all, fileName, outputFormat, dtype=fields_type(max_modifiers))
        if not r:
            QMessageBox.warning(None, cfg.programName, msg, QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)


def export_events_as_textgrid(self) -> None:
    """
    * select observations
    * select subjects, behaviors and time interval
    * export state events of selected observations as Praat textgrid
    """

    _, selected_observations = select_observations.select_observations2(self, mode=cfg.MULTIPLE, windows_title="")

    if not selected_observations:
        return

    # check if coded behaviors are defined in ethogram
    if project_functions.check_coded_behaviors_in_obs_list(self.pj, selected_observations):
        return

    # check if state events are paired
    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
    if not_ok or not selected_observations:
        return

    max_obs_length, _ = observation_operations.observation_length(self.pj, selected_observations)

    # exit with message if events do not have timestamp
    if max_obs_length.is_nan():
        QMessageBox.critical(
            None,
            cfg.programName,
            ("This function is not available for observations with events that do not have timestamp"),
            QMessageBox.Ok | QMessageBox.Default,
            QMessageBox.NoButton,
        )
        return

    start_coding, end_coding, _ = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], selected_observations)

    start_interval, end_interval = observation_operations.time_intervals_range(self.pj[cfg.OBSERVATIONS], selected_observations)

    parameters = select_subj_behav.choose_obs_subj_behav_category(
        self,
        selected_observations,
        start_coding=start_coding,
        end_coding=end_coding,
        # start_interval=start_interval,
        # end_interval=end_interval,
        start_interval=None,
        end_interval=None,
        show_include_modifiers=False,
        show_exclude_non_coded_behaviors=False,
        maxTime=max_obs_length,
        n_observations=len(selected_observations),
    )
    if parameters == {}:
        return
    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to export")
        return

    export_dir = QFileDialog.getExistingDirectory(
        self, "Export events as Praat TextGrid", os.path.expanduser("~"), options=QFileDialog.ShowDirsOnly
    )
    if not export_dir:
        return

    mem_command = ""

    # see https://www.fon.hum.uva.nl/praat/manual/TextGrid_file_formats.html

    interval_subject_header = (
        "    item [{subject_index}]:\n"
        '        class = "IntervalTier"\n'
        '        name = "{subject}"\n'
        "        xmin = 0.0\n"
        "        xmax = {intervalsMax}\n"
        "        intervals: size = {intervalsSize}\n"
    )

    interval_template = '        intervals [{count}]:\n            xmin = {xmin}\n            xmax = {xmax}\n            text = "{name}"\n'

    point_subject_header = (
        "    item [{subject_index}]:\n"
        '        class = "TextTier"\n'
        '        name = "{subject}"\n'
        "        xmin = {intervalsMin}\n"
        "        xmax = {intervalsMax}\n"
        "        points: size = {intervalsSize}\n"
    )

    point_template = '        points [{count}]:\n            number = {number}\n            mark = "{mark}"\n'

    # widget for results
    self.results = dialog.Results_dialog()
    self.results.setWindowTitle(f"{cfg.programName} - Export events as Praat TextGrid")
    self.results.show()

    ok, msg, db_connector = db_functions.load_aggregated_events_in_db(
        self.pj, parameters[cfg.SELECTED_SUBJECTS], selected_observations, parameters[cfg.SELECTED_BEHAVIORS]
    )

    if db_connector is None:
        logging.critical("Error when loading aggregated events in DB")
        return

    cursor = db_connector.cursor()

    file_count: int = 0

    for obs_id in selected_observations:
        if parameters["time"] == cfg.TIME_EVENTS:
            start_coding, end_coding, coding_duration = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], [obs_id])
            if start_coding is None and end_coding is None:  # no events
                self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have events.")
                QApplication.processEvents()
                continue

            if math.isnan(start_coding) or math.isnan(end_coding):  # obs with no timestamp
                self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have timestamp.")
                QApplication.processEvents()
                continue

            min_time = float(start_coding)
            max_time = float(end_coding)

        if parameters["time"] == cfg.TIME_FULL_OBS:
            if self.pj[cfg.OBSERVATIONS][obs_id][cfg.TYPE] == cfg.MEDIA:
                max_media_duration, _ = observation_operations.media_duration(self.pj[cfg.OBSERVATIONS], [obs_id])
                min_time = float(0)
                max_time = float(max_media_duration)
                coding_duration = max_media_duration

            if self.pj[cfg.OBSERVATIONS][obs_id][cfg.TYPE] in (cfg.LIVE, cfg.IMAGES):
                start_coding, end_coding, coding_duration = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], [obs_id])
                if start_coding is None and end_coding is None:  # no events
                    self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have events.")
                    QApplication.processEvents()
                    continue
                if math.isnan(start_coding) or math.isnan(end_coding):  # obs with no timestamp
                    self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have timestamp.")
                    QApplication.processEvents()
                    continue

                min_time = float(start_coding)
                max_time = float(end_coding)

        if parameters["time"] == cfg.TIME_ARBITRARY_INTERVAL:
            min_time = float(parameters[cfg.START_TIME])
            max_time = float(parameters[cfg.END_TIME])

        if parameters["time"] == cfg.TIME_OBS_INTERVAL:
            max_media_duration, _ = observation_operations.media_duration(self.pj[cfg.OBSERVATIONS], [obs_id])
            obs_interval = self.pj[cfg.OBSERVATIONS][obs_id].get(cfg.OBSERVATION_TIME_INTERVAL, [0, 0])
            offset = float(self.pj[cfg.OBSERVATIONS][obs_id][cfg.TIME_OFFSET])
            min_time = float(obs_interval[0]) + offset
            # Use max media duration for max time if no interval is defined (=0)
            max_time = float(obs_interval[1]) + offset if obs_interval[1] != 0 else float(max_media_duration)

        # delete events outside time interval
        cursor.execute(
            "DELETE FROM aggregated_events WHERE observation = ? AND (start < ? AND stop < ?) OR (start > ? AND stop > ?)",
            (
                obs_id,
                min_time,
                min_time,
                max_time,
                max_time,
            ),
        )
        cursor.execute(
            "UPDATE aggregated_events SET start = ? WHERE observation = ? AND start < ? AND stop BETWEEN ? AND ?",
            (
                min_time,
                obs_id,
                min_time,
                min_time,
                max_time,
            ),
        )
        cursor.execute(
            "UPDATE aggregated_events SET stop = ? WHERE observation = ? AND stop > ? AND start BETWEEN ? AND ?",
            (
                max_time,
                obs_id,
                max_time,
                min_time,
                max_time,
            ),
        )
        cursor.execute(
            "UPDATE aggregated_events SET start = ?, stop = ? WHERE observation = ? AND start < ? AND stop > ?",
            (
                min_time,
                max_time,
                obs_id,
                min_time,
                max_time,
            ),
        )

        next_obs: bool = False

        """
        total_media_duration = round(
            observation_operations.observation_total_length(self.pj[cfg.OBSERVATIONS][obs_id]), 3
        )
        """

        cursor.execute(
            (
                "SELECT COUNT(DISTINCT subject) FROM aggregated_events "
                "WHERE observation = ? AND subject IN ({}) AND type = 'STATE' ".format(
                    ",".join(["?"] * len(parameters[cfg.SELECTED_SUBJECTS]))
                )
            ),
            [obs_id] + parameters[cfg.SELECTED_SUBJECTS],
        )

        subjectsNum = int(cursor.fetchone()[0])
        """subjectsMin = min_time"""
        subjectsMax = max_time

        out = (
            'File type = "ooTextFile"\n'
            'Object class = "TextGrid"\n'
            "\n"
            f"xmin = 0.0\n"
            f"xmax = {subjectsMax}\n"
            "tiers? <exists>\n"
            f"size = {subjectsNum}\n"
            "item []:\n"
        )

        subject_index = 0
        for subject in parameters[cfg.SELECTED_SUBJECTS]:
            if subject not in [
                x[cfg.EVENT_SUBJECT_FIELD_IDX] if x[cfg.EVENT_SUBJECT_FIELD_IDX] else cfg.NO_FOCAL_SUBJECT
                for x in self.pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS]
            ]:
                continue

            intervalsMin, intervalsMax = min_time, max_time

            # STATE events
            cursor.execute(
                (
                    "SELECT start, stop, behavior FROM aggregated_events "
                    "WHERE observation = ? AND subject = ? AND type = 'STATE' ORDER BY start"
                ),
                (obs_id, subject),
            )

            rows = [
                {"start": util.float2decimal(r["start"]), "stop": util.float2decimal(r["stop"]), "code": r["behavior"]}
                for r in cursor.fetchall()
            ]
            if not rows:
                continue

            out += interval_subject_header

            count = 0

            # check if 1st behavior starts at the beginning
            if rows[0]["start"] > 0:
                count += 1
                out += interval_template.format(count=count, name="null", xmin=0.0, xmax=rows[0]["start"])

            for idx, row in enumerate(rows):
                # check if events are overlapping
                if (idx + 1 < len(rows)) and (row["stop"] > rows[idx + 1]["start"]):
                    self.results.ptText.appendHtml(
                        (
                            f"The events overlap for subject <b>{subject}</b> in the observation <b>{obs_id}</b>. "
                            "It is not possible to create the Praat TextGrid file."
                        )
                    )
                    QApplication.processEvents()

                    next_obs = True
                    break

                count += 1

                if (idx + 1 < len(rows)) and (rows[idx + 1]["start"] - dec("0.001") <= row["stop"] < rows[idx + 1]["start"]):
                    xmax = rows[idx + 1]["start"]
                else:
                    xmax = row["stop"]

                out += interval_template.format(count=count, name=row["code"], xmin=row["start"], xmax=xmax)

                # check if no behavior
                if (idx + 1 < len(rows)) and (row["stop"] < rows[idx + 1]["start"] - dec("0.001")):
                    count += 1
                    out += interval_template.format(
                        count=count,
                        name="null",
                        xmin=row["stop"],
                        xmax=rows[idx + 1]["start"],
                    )

            if next_obs:
                break

            # check if last event ends at the end of media file
            if rows[-1]["stop"] < max_time:
                count += 1
                out += interval_template.format(count=count, name="null", xmin=rows[-1]["stop"], xmax=max_time)

            # add info
            subject_index += 1
            out = out.format(
                subject_index=subject_index,
                subject=subject,
                intervalsSize=count,
                intervalsMin=intervalsMin,
                intervalsMax=intervalsMax,
            )

            # POINT events
            cursor.execute(
                ("SELECT start, behavior FROM aggregated_events WHERE observation = ? AND subject = ? AND type = 'POINT' ORDER BY start"),
                (obs_id, subject),
            )

            rows = [{"start": util.float2decimal(r["start"]), "code": r["behavior"]} for r in cursor.fetchall()]
            if not rows:
                continue

            out += point_subject_header

            count = 0

            for idx, row in enumerate(rows):
                count += 1
                out += point_template.format(count=count, mark=row["code"], number=row["start"])

            # add info
            subject_index += 1
            out = out.format(
                subject_index=subject_index,
                subject=subject,
                intervalsSize=count,
                intervalsMin=intervalsMin,
                intervalsMax=intervalsMax,
            )

        if next_obs:
            continue

        # check if file already exists
        if mem_command != cfg.OVERWRITE_ALL and pl.Path(f"{pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid").is_file():
            if mem_command == cfg.SKIP_ALL:
                continue
            mem_command = dialog.MessageDialog(
                cfg.programName,
                f"The file <b>{pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid</b> already exists.",
                [cfg.OVERWRITE, cfg.OVERWRITE_ALL, cfg.SKIP, cfg.SKIP_ALL, cfg.CANCEL],
            )
            if mem_command == cfg.CANCEL:
                return
            if mem_command in (cfg.SKIP, cfg.SKIP_ALL):
                continue

        try:
            with open(f"{pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid", "w") as f:
                f.write(out)
            file_count += 1
            self.results.ptText.appendHtml(f"File {pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid was created.")
            QApplication.processEvents()
        except Exception:
            self.results.ptText.appendHtml(f"The file {pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid can not be created.")
            QApplication.processEvents()

    self.results.ptText.appendHtml(f"Done.  {file_count} file(s) were created in {export_dir}.")
    QApplication.processEvents()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'param_panel.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QDialog, QFrame,
    QHBoxLayout, QLabel, QListWidget, QListWidgetItem,
    QPushButton, QRadioButton, QSizePolicy, QSpacerItem,
    QSpinBox, QVBoxLayout, QWidget)

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(1037, 890)
        self.verticalLayout_4 = QVBoxLayout(Dialog)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.lbSubjects = QLabel(Dialog)
        self.lbSubjects.setObjectName(u"lbSubjects")

        self.verticalLayout_2.addWidget(self.lbSubjects)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.pbSelectAllSubjects = QPushButton(Dialog)
        self.pbSelectAllSubjects.setObjectName(u"pbSelectAllSubjects")

        self.horizontalLayout_3.addWidget(self.pbSelectAllSubjects)

        self.pbUnselectAllSubjects = QPushButton(Dialog)
        self.pbUnselectAllSubjects.setObjectName(u"pbUnselectAllSubjects")

        self.horizontalLayout_3.addWidget(self.pbUnselectAllSubjects)

        self.pbReverseSubjectsSelection = QPushButton(Dialog)
        self.pbReverseSubjectsSelection.setObjectName(u"pbReverseSubjectsSelection")

        self.horizontalLayout_3.addWidget(self.pbReverseSubjectsSelection)

        self.horizontalSpacer_4 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer_4)


        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.lwSubjects = QListWidget(Dialog)
        self.lwSubjects.setObjectName(u"lwSubjects")

        self.verticalLayout_2.addWidget(self.lwSubjects)

        self.lbBehaviors = QLabel(Dialog)
        self.lbBehaviors.setObjectName(u"lbBehaviors")

        self.verticalLayout_2.addWidget(self.lbBehaviors)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.pbSelectAllBehaviors = QPushButton(Dialog)
        self.pbSelectAllBehaviors.setObjectName(u"pbSelectAllBehaviors")

        self.horizontalLayout_4.addWidget(self.pbSelectAllBehaviors)

        self.pbUnselectAllBehaviors = QPushButton(Dialog)
        self.pbUnselectAllBehaviors.setObjectName(u"pbUnselectAllBehaviors")

        self.horizontalLayout_4.addWidget(self.pbUnselectAllBehaviors)

        self.pbReverseBehaviorsSelection = QPushButton(Dialog)
        self.pbReverseBehaviorsSelection.setObjectName(u"pbReverseBehaviorsSelection")

        self.horizontalLayout_4.addWidget(self.pbReverseBehaviorsSelection)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer_5)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        self.lwBehaviors = QListWidget(Dialog)
        self.lwBehaviors.setObjectName(u"lwBehaviors")

        self.verticalLayout_2.addWidget(self.lwBehaviors)

        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.cbIncludeModifiers = QCheckBox(Dialog)
        self.cbIncludeModifiers.setObjectName(u"cbIncludeModifiers")

        self.horizontalLayout_9.addWidget(self.cbIncludeModifiers)

        self.cb_exclude_non_coded_modifiers = QCheckBox(Dialog)
        self.cb_exclude_non_coded_modifiers.setObjectName(u"cb_exclude_non_coded_modifiers")
        self.cb_exclude_non_coded_modifiers.setChecked(True)

        self.horizontalLayout_9.addWidget(self.cb_exclude_non_coded_modifiers)

        self.cbExcludeBehaviors = QCheckBox(Dialog)
        self.cbExcludeBehaviors.setObjectName(u"cbExcludeBehaviors")

        self.horizontalLayout_9.addWidget(self.cbExcludeBehaviors)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_9.addItem(self.horizontalSpacer_7)


        self.verticalLayout_2.addLayout(self.horizontalLayout_9)

        self.frm_time_bin_size = QFrame(Dialog)
        self.frm_time_bin_size.setObjectName(u"frm_time_bin_size")
        self.frm_time_bin_size.setFrameShape(QFrame.StyledPanel)
        self.frm_time_bin_size.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_8 = QHBoxLayout(self.frm_time_bin_size)
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.lb_time_bin_size = QLabel(self.frm_time_bin_size)
        self.lb_time_bin_size.setObjectName(u"lb_time_bin_size")

        self.horizontalLayout_7.addWidget(self.lb_time_bin_size)

        self.sb_time_bin_size = QSpinBox(self.frm_time_bin_size)
        self.sb_time_bin_size.setObjectName(u"sb_time_bin_size")
        self.sb_time_bin_size.setMaximum(86400)
        self.sb_time_bin_size.setSingleStep(10)

        self.horizontalLayout_7.addWidget(self.sb_time_bin_size)

        self.horizontalSpacer_8 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer_8)


        self.horizontalLayout_8.addLayout(self.horizontalLayout_7)


        self.verticalLayout_2.addWidget(self.frm_time_bin_size)

        self.frm_time = QFrame(Dialog)
        self.frm_time.setObjectName(u"frm_time")
        self.frm_time.setFrameShape(QFrame.StyledPanel)
        self.frm_time.setFrameShadow(QFrame.Raised)
        self.verticalLayout_3 = QVBoxLayout(self.frm_time)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.lb_time_interval = QLabel(self.frm_time)
        self.lb_time_interval.setObjectName(u"lb_time_interval")

        self.verticalLayout_3.addWidget(self.lb_time_interval)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.rb_observed_events = QRadioButton(self.frm_time)
        self.rb_observed_events.setObjectName(u"rb_observed_events")

        self.horizontalLayout_5.addWidget(self.rb_observed_events)

        self.rb_user_defined = QRadioButton(self.frm_time)
        self.rb_user_defined.setObjectName(u"rb_user_defined")

        self.horizontalLayout_5.addWidget(self.rb_user_defined)

        self.rb_obs_interval = QRadioButton(self.frm_time)
        self.rb_obs_interval.setObjectName(u"rb_obs_interval")

        self.horizontalLayout_5.addWidget(self.rb_obs_interval)

        self.rb_media_duration = QRadioButton(self.frm_time)
        self.rb_media_duration.setObjectName(u"rb_media_duration")
        self.rb_media_duration.setCheckable(True)
        self.rb_media_duration.setChecked(False)

        self.horizontalLayout_5.addWidget(self.rb_media_duration)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_5.addItem(self.horizontalSpacer_3)


        self.verticalLayout_3.addLayout(self.horizontalLayout_5)

        self.frm_time_interval = QFrame(self.frm_time)
        self.frm_time_interval.setObjectName(u"frm_time_interval")
        self.frm_time_interval.setFrameShape(QFrame.StyledPanel)
        self.frm_time_interval.setFrameShadow(QFrame.Raised)
        self.verticalLayout = QVBoxLayout(self.frm_time_interval)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.lbStartTime = QLabel(self.frm_time_interval)
        self.lbStartTime.setObjectName(u"lbStartTime")

        self.horizontalLayout.addWidget(self.lbStartTime)

        self.label_2 = QLabel(self.frm_time_interval)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout.addWidget(self.label_2)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.lbEndTime = QLabel(self.frm_time_interval)
        self.lbEndTime.setObjectName(u"lbEndTime")

        self.horizontalLayout_6.addWidget(self.lbEndTime)

        self.label_3 = QLabel(self.frm_time_interval)
        self.label_3.setObjectName(u"label_3")

        self.horizontalLayout_6.addWidget(self.label_3)

        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_6.addItem(self.horizontalSpacer_6)


        self.verticalLayout.addLayout(self.horizontalLayout_6)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)


        self.verticalLayout_3.addWidget(self.frm_time_interval)


        self.verticalLayout_2.addWidget(self.frm_time)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.pbCancel = QPushButton(Dialog)
        self.pbCancel.setObjectName(u"pbCancel")

        self.horizontalLayout_2.addWidget(self.pbCancel)

        self.pbOK = QPushButton(Dialog)
        self.pbOK.setObjectName(u"pbOK")

        self.horizontalLayout_2.addWidget(self.pbOK)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)


        self.verticalLayout_4.addLayout(self.verticalLayout_2)


        self.retranslateUi(Dialog)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Parameters", None))
        self.lbSubjects.setText(QCoreApplication.translate("Dialog", u"Subjects", None))
        self.pbSelectAllSubjects.setText(QCoreApplication.translate("Dialog", u"Select all", None))
        self.pbUnselectAllSubjects.setText(QCoreApplication.translate("Dialog", u"Unselect all", None))
        self.pbReverseSubjectsSelection.setText(QCoreApplication.translate("Dialog", u"Reverse selection", None))
        self.lbBehaviors.setText(QCoreApplication.translate("Dialog", u"Behaviors", None))
        self.pbSelectAllBehaviors.setText(QCoreApplication.translate("Dialog", u"Select all", None))
        self.pbUnselectAllBehaviors.setText(QCoreApplication.translate("Dialog", u"Unselect all", None))
        self.pbReverseBehaviorsSelection.setText(QCoreApplication.translate("Dialog", u"Reverse selection", None))
        self.cbIncludeModifiers.setText(QCoreApplication.translate("Dialog", u"Include modifiers", None))
        self.cb_exclude_non_coded_modifiers.setText(QCoreApplication.translate("Dialog", u"Exclude non coded modifiers", None))
        self.cbExcludeBehaviors.setText(QCoreApplication.translate("Dialog", u"Exclude behaviors without events", None))
        self.lb_time_bin_size.setText(QCoreApplication.translate("Dialog", u"Time bin size (s)", None))
        self.lb_time_interval.setText(QCoreApplication.translate("Dialog", u"Time interval", None))
        self.rb_observed_events.setText(QCoreApplication.translate("Dialog", u"Observed events", None))
        self.rb_user_defined.setText(QCoreApplication.translate("Dialog", u"User defined", None))
        self.rb_obs_interval.setText(QCoreApplication.translate("Dialog", u"Interval of observation", None))
        self.rb_media_duration.setText(QCoreApplication.translate("Dialog", u"Media file(s) duration", None))
        self.lbStartTime.setText(QCoreApplication.translate("Dialog", u"Start time", None))
        self.label_2.setText("")
        self.lbEndTime.setText(QCoreApplication.translate("Dialog", u"End time", None))
        self.label_3.setText("")
        self.pbCancel.setText(QCoreApplication.translate("Dialog", u"Cancel", None))
        self.pbOK.setText(QCoreApplication.translate("Dialog", u"OK", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QDialog,
    QDoubleSpinBox, QFormLayout, QGridLayout, QHBoxLayout,
    QLabel, QLineEdit, QListWidget, QListWidgetItem,
    QPlainTextEdit, QPushButton, QSizePolicy, QSpacerItem,
    QSpinBox, QTabWidget, QVBoxLayout, QWidget)

class Ui_prefDialog(object):
    def setupUi(self, prefDialog):
        if not prefDialog.objectName():
            prefDialog.setObjectName(u"prefDialog")
        prefDialog.setWindowModality(Qt.WindowModality.WindowModal)
        prefDialog.resize(904, 554)
        self.gridLayout = QGridLayout(prefDialog)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.tabWidget = QTabWidget(prefDialog)
        self.tabWidget.setObjectName(u"tabWidget")
        self.tabWidget.setEnabled(True)
        self.tab_project = QWidget()
        self.tab_project.setObjectName(u"tab_project")
        self.verticalLayout_5 = QVBoxLayout(self.tab_project)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.horizontalLayout_14 = QHBoxLayout()
        self.horizontalLayout_14.setObjectName(u"horizontalLayout_14")
        self.label = QLabel(self.tab_project)
        self.label.setObjectName(u"label")

        self.horizontalLayout_14.addWidget(self.label)

        self.cbTimeFormat = QComboBox(self.tab_project)
        self.cbTimeFormat.addItem("")
        self.cbTimeFormat.addItem("")
        self.cbTimeFormat.setObjectName(u"cbTimeFormat")

        self.horizontalLayout_14.addWidget(self.cbTimeFormat)


        self.verticalLayout_5.addLayout(self.horizontalLayout_14)

        self.horizontalLayout_15 = QHBoxLayout()
        self.horizontalLayout_15.setObjectName(u"horizontalLayout_15")
        self.label_6 = QLabel(self.tab_project)
        self.label_6.setObjectName(u"label_6")

        self.horizontalLayout_15.addWidget(self.label_6)

        self.sbAutomaticBackup = QSpinBox(self.tab_project)
        self.sbAutomaticBackup.setObjectName(u"sbAutomaticBackup")
        self.sbAutomaticBackup.setMinimum(-10000)
        self.sbAutomaticBackup.setMaximum(10000)
        self.sbAutomaticBackup.setValue(10)

        self.horizontalLayout_15.addWidget(self.sbAutomaticBackup)


        self.verticalLayout_5.addLayout(self.horizontalLayout_15)

        self.horizontalLayout_13 = QHBoxLayout()
        self.horizontalLayout_13.setObjectName(u"horizontalLayout_13")
        self.label_3 = QLabel(self.tab_project)
        self.label_3.setObjectName(u"label_3")

        self.horizontalLayout_13.addWidget(self.label_3)

        self.leSeparator = QLineEdit(self.tab_project)
        self.leSeparator.setObjectName(u"leSeparator")

        self.horizontalLayout_13.addWidget(self.leSeparator)


        self.verticalLayout_5.addLayout(self.horizontalLayout_13)

        self.cbCheckForNewVersion = QCheckBox(self.tab_project)
        self.cbCheckForNewVersion.setObjectName(u"cbCheckForNewVersion")

        self.verticalLayout_5.addWidget(self.cbCheckForNewVersion)

        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setObjectName(u"horizontalLayout_11")
        self.lb_hwdec = QLabel(self.tab_project)
        self.lb_hwdec.setObjectName(u"lb_hwdec")

        self.horizontalLayout_11.addWidget(self.lb_hwdec)

        self.cb_hwdec = QComboBox(self.tab_project)
        self.cb_hwdec.setObjectName(u"cb_hwdec")

        self.horizontalLayout_11.addWidget(self.cb_hwdec)


        self.verticalLayout_5.addLayout(self.horizontalLayout_11)

        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.lb_project_file_indent = QLabel(self.tab_project)
        self.lb_project_file_indent.setObjectName(u"lb_project_file_indent")

        self.horizontalLayout_9.addWidget(self.lb_project_file_indent)

        self.combo_project_file_indentation = QComboBox(self.tab_project)
        self.combo_project_file_indentation.setObjectName(u"combo_project_file_indentation")

        self.horizontalLayout_9.addWidget(self.combo_project_file_indentation)


        self.verticalLayout_5.addLayout(self.horizontalLayout_9)

        self.cb_check_integrity_at_opening = QCheckBox(self.tab_project)
        self.cb_check_integrity_at_opening.setObjectName(u"cb_check_integrity_at_opening")

        self.verticalLayout_5.addWidget(self.cb_check_integrity_at_opening)

        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_5.addItem(self.verticalSpacer_2)

        self.tabWidget.addTab(self.tab_project, "")
        self.tab_observations = QWidget()
        self.tab_observations.setObjectName(u"tab_observations")
        self.verticalLayout = QVBoxLayout(self.tab_observations)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.label_4 = QLabel(self.tab_observations)
        self.label_4.setObjectName(u"label_4")

        self.horizontalLayout_4.addWidget(self.label_4)

        self.sbffSpeed = QSpinBox(self.tab_observations)
        self.sbffSpeed.setObjectName(u"sbffSpeed")
        self.sbffSpeed.setMinimum(0)
        self.sbffSpeed.setMaximum(10000)
        self.sbffSpeed.setValue(10)

        self.horizontalLayout_4.addWidget(self.sbffSpeed)


        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.cb_adapt_fast_jump = QCheckBox(self.tab_observations)
        self.cb_adapt_fast_jump.setObjectName(u"cb_adapt_fast_jump")

        self.verticalLayout.addWidget(self.cb_adapt_fast_jump)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_5 = QLabel(self.tab_observations)
        self.label_5.setObjectName(u"label_5")

        self.horizontalLayout_5.addWidget(self.label_5)

        self.sbSpeedStep = QDoubleSpinBox(self.tab_observations)
        self.sbSpeedStep.setObjectName(u"sbSpeedStep")
        self.sbSpeedStep.setDecimals(1)
        self.sbSpeedStep.setMinimum(0.100000000000000)
        self.sbSpeedStep.setMaximum(10.000000000000000)
        self.sbSpeedStep.setSingleStep(0.100000000000000)
        self.sbSpeedStep.setValue(0.100000000000000)

        self.horizontalLayout_5.addWidget(self.sbSpeedStep)


        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.label_2 = QLabel(self.tab_observations)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout_6.addWidget(self.label_2)

        self.sbRepositionTimeOffset = QSpinBox(self.tab_observations)
        self.sbRepositionTimeOffset.setObjectName(u"sbRepositionTimeOffset")
        self.sbRepositionTimeOffset.setMinimum(-10000)
        self.sbRepositionTimeOffset.setMaximum(10000)
        self.sbRepositionTimeOffset.setValue(-3)

        self.horizontalLayout_6.addWidget(self.sbRepositionTimeOffset)


        self.verticalLayout.addLayout(self.horizontalLayout_6)

        self.cbConfirmSound = QCheckBox(self.tab_observations)
        self.cbConfirmSound.setObjectName(u"cbConfirmSound")

        self.verticalLayout.addWidget(self.cbConfirmSound)

        self.cbCloseSameEvent = QCheckBox(self.tab_observations)
        self.cbCloseSameEvent.setObjectName(u"cbCloseSameEvent")

        self.verticalLayout.addWidget(self.cbCloseSameEvent)

        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.label_8 = QLabel(self.tab_observations)
        self.label_8.setObjectName(u"label_8")

        self.horizontalLayout_8.addWidget(self.label_8)

        self.sbBeepEvery = QSpinBox(self.tab_observations)
        self.sbBeepEvery.setObjectName(u"sbBeepEvery")

        self.horizontalLayout_8.addWidget(self.sbBeepEvery)


        self.verticalLayout.addLayout(self.horizontalLayout_8)

        self.cb_display_subtitles = QCheckBox(self.tab_observations)
        self.cb_display_subtitles.setObjectName(u"cb_display_subtitles")

        self.verticalLayout.addWidget(self.cb_display_subtitles)

        self.cbTrackingCursorAboveEvent = QCheckBox(self.tab_observations)
        self.cbTrackingCursorAboveEvent.setObjectName(u"cbTrackingCursorAboveEvent")

        self.verticalLayout.addWidget(self.cbTrackingCursorAboveEvent)

        self.cbAlertNoFocalSubject = QCheckBox(self.tab_observations)
        self.cbAlertNoFocalSubject.setObjectName(u"cbAlertNoFocalSubject")

        self.verticalLayout.addWidget(self.cbAlertNoFocalSubject)

        self.cb_pause_before_addevent = QCheckBox(self.tab_observations)
        self.cb_pause_before_addevent.setObjectName(u"cb_pause_before_addevent")

        self.verticalLayout.addWidget(self.cb_pause_before_addevent)

        self.verticalSpacer_4 = QSpacerItem(20, 391, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer_4)

        self.tabWidget.addTab(self.tab_observations, "")
        self.tab_analysis_plugins = QWidget()
        self.tab_analysis_plugins.setObjectName(u"tab_analysis_plugins")
        self.horizontalLayout_17 = QHBoxLayout(self.tab_analysis_plugins)
        self.horizontalLayout_17.setObjectName(u"horizontalLayout_17")
        self.verticalLayout_11 = QVBoxLayout()
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.label_13 = QLabel(self.tab_analysis_plugins)
        self.label_13.setObjectName(u"label_13")

        self.verticalLayout_11.addWidget(self.label_13)

        self.lv_all_plugins = QListWidget(self.tab_analysis_plugins)
        self.lv_all_plugins.setObjectName(u"lv_all_plugins")

        self.verticalLayout_11.addWidget(self.lv_all_plugins)

        self.label_15 = QLabel(self.tab_analysis_plugins)
        self.label_15.setObjectName(u"label_15")

        self.verticalLayout_11.addWidget(self.label_15)

        self.horizontalLayout_16 = QHBoxLayout()
        self.horizontalLayout_16.setObjectName(u"horizontalLayout_16")
        self.le_personal_plugins_dir = QLineEdit(self.tab_analysis_plugins)
        self.le_personal_plugins_dir.setObjectName(u"le_personal_plugins_dir")
        self.le_personal_plugins_dir.setReadOnly(True)

        self.horizontalLayout_16.addWidget(self.le_personal_plugins_dir)

        self.pb_browse_plugins_dir = QPushButton(self.tab_analysis_plugins)
        self.pb_browse_plugins_dir.setObjectName(u"pb_browse_plugins_dir")

        self.horizontalLayout_16.addWidget(self.pb_browse_plugins_dir)


        self.verticalLayout_11.addLayout(self.horizontalLayout_16)

        self.lw_personal_plugins = QListWidget(self.tab_analysis_plugins)
        self.lw_personal_plugins.setObjectName(u"lw_personal_plugins")

        self.verticalLayout_11.addWidget(self.lw_personal_plugins)


        self.horizontalLayout_17.addLayout(self.verticalLayout_11)

        self.verticalLayout_12 = QVBoxLayout()
        self.verticalLayout_12.setObjectName(u"verticalLayout_12")
        self.label_14 = QLabel(self.tab_analysis_plugins)
        self.label_14.setObjectName(u"label_14")

        self.verticalLayout_12.addWidget(self.label_14)

        self.pte_plugin_description = QPlainTextEdit(self.tab_analysis_plugins)
        self.pte_plugin_description.setObjectName(u"pte_plugin_description")
        self.pte_plugin_description.setReadOnly(True)

        self.verticalLayout_12.addWidget(self.pte_plugin_description)


        self.horizontalLayout_17.addLayout(self.verticalLayout_12)

        self.tabWidget.addTab(self.tab_analysis_plugins, "")
        self.tab_ffmpeg = QWidget()
        self.tab_ffmpeg.setObjectName(u"tab_ffmpeg")
        self.verticalLayout_4 = QVBoxLayout(self.tab_ffmpeg)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.lbFFmpegPath = QLabel(self.tab_ffmpeg)
        self.lbFFmpegPath.setObjectName(u"lbFFmpegPath")
        self.lbFFmpegPath.setScaledContents(False)
        self.lbFFmpegPath.setWordWrap(True)

        self.verticalLayout_3.addWidget(self.lbFFmpegPath)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")

        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.lbFFmpegCacheDir = QLabel(self.tab_ffmpeg)
        self.lbFFmpegCacheDir.setObjectName(u"lbFFmpegCacheDir")

        self.horizontalLayout_3.addWidget(self.lbFFmpegCacheDir)

        self.leFFmpegCacheDir = QLineEdit(self.tab_ffmpeg)
        self.leFFmpegCacheDir.setObjectName(u"leFFmpegCacheDir")

        self.horizontalLayout_3.addWidget(self.leFFmpegCacheDir)

        self.pbBrowseFFmpegCacheDir = QPushButton(self.tab_ffmpeg)
        self.pbBrowseFFmpegCacheDir.setObjectName(u"pbBrowseFFmpegCacheDir")

        self.horizontalLayout_3.addWidget(self.pbBrowseFFmpegCacheDir)


        self.verticalLayout_3.addLayout(self.horizontalLayout_3)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_3.addItem(self.verticalSpacer)


        self.verticalLayout_4.addLayout(self.verticalLayout_3)

        self.tabWidget.addTab(self.tab_ffmpeg, "")
        self.tab_spectro = QWidget()
        self.tab_spectro.setObjectName(u"tab_spectro")
        self.verticalLayout_8 = QVBoxLayout(self.tab_spectro)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.label_7 = QLabel(self.tab_spectro)
        self.label_7.setObjectName(u"label_7")

        self.horizontalLayout_7.addWidget(self.label_7)

        self.cbSpectrogramColorMap = QComboBox(self.tab_spectro)
        self.cbSpectrogramColorMap.setObjectName(u"cbSpectrogramColorMap")

        self.horizontalLayout_7.addWidget(self.cbSpectrogramColorMap)


        self.verticalLayout_8.addLayout(self.horizontalLayout_7)

        self.horizontalLayout_10 = QHBoxLayout()
        self.horizontalLayout_10.setObjectName(u"horizontalLayout_10")
        self.label_12 = QLabel(self.tab_spectro)
        self.label_12.setObjectName(u"label_12")

        self.horizontalLayout_10.addWidget(self.label_12)

        self.sb_time_interval = QSpinBox(self.tab_spectro)
        self.sb_time_interval.setObjectName(u"sb_time_interval")
        self.sb_time_interval.setMinimum(2)
        self.sb_time_interval.setMaximum(360)
        self.sb_time_interval.setValue(10)

        self.horizontalLayout_10.addWidget(self.sb_time_interval)


        self.verticalLayout_8.addLayout(self.horizontalLayout_10)

        self.verticalSpacer_3 = QSpacerItem(20, 319, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_8.addItem(self.verticalSpacer_3)

        self.tabWidget.addTab(self.tab_spectro, "")
        self.tab_colors = QWidget()
        self.tab_colors.setObjectName(u"tab_colors")
        self.verticalLayout_10 = QVBoxLayout(self.tab_colors)
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.horizontalLayout_12 = QHBoxLayout()
        self.horizontalLayout_12.setObjectName(u"horizontalLayout_12")
        self.verticalLayout_6 = QVBoxLayout()
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.label_10 = QLabel(self.tab_colors)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setOpenExternalLinks(True)

        self.verticalLayout_6.addWidget(self.label_10)

        self.te_behav_colors = QPlainTextEdit(self.tab_colors)
        self.te_behav_colors.setObjectName(u"te_behav_colors")

        self.verticalLayout_6.addWidget(self.te_behav_colors)

        self.pb_reset_behav_colors = QPushButton(self.tab_colors)
        self.pb_reset_behav_colors.setObjectName(u"pb_reset_behav_colors")

        self.verticalLayout_6.addWidget(self.pb_reset_behav_colors)


        self.horizontalLayout_12.addLayout(self.verticalLayout_6)

        self.verticalLayout_9 = QVBoxLayout()
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.label_11 = QLabel(self.tab_colors)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setOpenExternalLinks(True)

        self.verticalLayout_9.addWidget(self.label_11)

        self.te_category_colors = QPlainTextEdit(self.tab_colors)
        self.te_category_colors.setObjectName(u"te_category_colors")

        self.verticalLayout_9.addWidget(self.te_category_colors)

        self.pb_reset_category_colors = QPushButton(self.tab_colors)
        self.pb_reset_category_colors.setObjectName(u"pb_reset_category_colors")

        self.verticalLayout_9.addWidget(self.pb_reset_category_colors)


        self.horizontalLayout_12.addLayout(self.verticalLayout_9)


        self.verticalLayout_10.addLayout(self.horizontalLayout_12)

        self.tabWidget.addTab(self.tab_colors, "")
        self.tab_interface = QWidget()
        self.tab_interface.setObjectName(u"tab_interface")
        self.verticalLayout_7 = QVBoxLayout(self.tab_interface)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.formLayout = QFormLayout()
        self.formLayout.setObjectName(u"formLayout")
        self.label_9 = QLabel(self.tab_interface)
        self.label_9.setObjectName(u"label_9")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label_9)

        self.sb_toolbar_icon_size = QSpinBox(self.tab_interface)
        self.sb_toolbar_icon_size.setObjectName(u"sb_toolbar_icon_size")
        self.sb_toolbar_icon_size.setMinimum(12)
        self.sb_toolbar_icon_size.setMaximum(128)
        self.sb_toolbar_icon_size.setValue(24)

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.sb_toolbar_icon_size)


        self.verticalLayout_7.addLayout(self.formLayout)

        self.verticalSpacer_5 = QSpacerItem(20, 386, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_7.addItem(self.verticalSpacer_5)

        self.tabWidget.addTab(self.tab_interface, "")

        self.verticalLayout_2.addWidget(self.tabWidget)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(241, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.pb_refresh = QPushButton(prefDialog)
        self.pb_refresh.setObjectName(u"pb_refresh")

        self.horizontalLayout_2.addWidget(self.pb_refresh)

        self.pbCancel = QPushButton(prefDialog)
        self.pbCancel.setObjectName(u"pbCancel")

        self.horizontalLayout_2.addWidget(self.pbCancel)

        self.pbOK = QPushButton(prefDialog)
        self.pbOK.setObjectName(u"pbOK")

        self.horizontalLayout_2.addWidget(self.pbOK)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)


        self.gridLayout.addLayout(self.verticalLayout_2, 0, 0, 1, 1)


        self.retranslateUi(prefDialog)

        self.tabWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(prefDialog)
    # setupUi

    def retranslateUi(self, prefDialog):
        prefDialog.setWindowTitle(QCoreApplication.translate("prefDialog", u"Preferences", None))
        self.label.setText(QCoreApplication.translate("prefDialog", u"Default project time format", None))
        self.cbTimeFormat.setItemText(0, QCoreApplication.translate("prefDialog", u"seconds", None))
        self.cbTimeFormat.setItemText(1, QCoreApplication.translate("prefDialog", u"hh:mm:ss.mss", None))

        self.label_6.setText(QCoreApplication.translate("prefDialog", u"Auto-save project every (minutes)", None))
        self.label_3.setText(QCoreApplication.translate("prefDialog", u"Separator for behavioural strings (events export)", None))
        self.leSeparator.setText(QCoreApplication.translate("prefDialog", u"|", None))
        self.cbCheckForNewVersion.setText(QCoreApplication.translate("prefDialog", u"Check for new version and news", None))
        self.lb_hwdec.setText(QCoreApplication.translate("prefDialog", u"MPV player hardware video decoding", None))
        self.lb_project_file_indent.setText(QCoreApplication.translate("prefDialog", u"Project file indentation type", None))
        self.cb_check_integrity_at_opening.setText(QCoreApplication.translate("prefDialog", u"Check project integrity when opening and saving project (recommended)", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_project), QCoreApplication.translate("prefDialog", u"Project", None))
        self.label_4.setText(QCoreApplication.translate("prefDialog", u"Fast forward/backward value (seconds)", None))
        self.cb_adapt_fast_jump.setText(QCoreApplication.translate("prefDialog", u"Adapt the fast forward/backward jump to playback speed", None))
        self.label_5.setText(QCoreApplication.translate("prefDialog", u"Playback speed step value", None))
        self.label_2.setText(QCoreApplication.translate("prefDialog", u"Time offset for video/audio reposition (seconds)", None))
        self.cbConfirmSound.setText(QCoreApplication.translate("prefDialog", u"Play sound when a key is pressed", None))
        self.cbCloseSameEvent.setText(QCoreApplication.translate("prefDialog", u"Close the same current event independently of modifiers", None))
        self.label_8.setText(QCoreApplication.translate("prefDialog", u"Beep every (seconds)", None))
        self.cb_display_subtitles.setText(QCoreApplication.translate("prefDialog", u"Display subtitles", None))
        self.cbTrackingCursorAboveEvent.setText(QCoreApplication.translate("prefDialog", u"Tracking cursor above current event", None))
        self.cbAlertNoFocalSubject.setText(QCoreApplication.translate("prefDialog", u"Alert if focal subject is not set", None))
        self.cb_pause_before_addevent.setText(QCoreApplication.translate("prefDialog", u"Pause media before \"Add event\" command", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_observations), QCoreApplication.translate("prefDialog", u"Observations", None))
        self.label_13.setText(QCoreApplication.translate("prefDialog", u"BORIS plugins", None))
        self.label_15.setText(QCoreApplication.translate("prefDialog", u"Personal plugins", None))
        self.pb_browse_plugins_dir.setText(QCoreApplication.translate("prefDialog", u"Browse", None))
        self.label_14.setText(QCoreApplication.translate("prefDialog", u"Plugin info", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_analysis_plugins), QCoreApplication.translate("prefDialog", u"Analysis plugins", None))
        self.lbFFmpegPath.setText(QCoreApplication.translate("prefDialog", u"FFmpeg path:", None))
        self.lbFFmpegCacheDir.setText(QCoreApplication.translate("prefDialog", u"FFmpeg cache directory", None))
        self.pbBrowseFFmpegCacheDir.setText(QCoreApplication.translate("prefDialog", u"...", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_ffmpeg), QCoreApplication.translate("prefDialog", u"FFmpeg framework", None))
        self.label_7.setText(QCoreApplication.translate("prefDialog", u"Spectrogram color map", None))
        self.label_12.setText(QCoreApplication.translate("prefDialog", u"Default time interval (s)", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_spectro), QCoreApplication.translate("prefDialog", u"Spectrogram/Wave form", None))
        self.label_10.setText(QCoreApplication.translate("prefDialog", u"<html><head/><body><p>List of colors for behaviors. See <a href=\"https://matplotlib.org/api/colors_api.html\"><span style=\" text-decoration: underline; color:#0000ff;\">matplotlib colors</span></a></p></body></html>", None))
        self.pb_reset_behav_colors.setText(QCoreApplication.translate("prefDialog", u"Reset colors to default", None))
        self.label_11.setText(QCoreApplication.translate("prefDialog", u"<html><head/><body><p>List of colors for behavioral categories. See <a href=\"https://matplotlib.org/api/colors_api.html\"><span style=\" text-decoration: underline; color:#0000ff;\">matplotlib colors</span></a></p></body></html>", None))
        self.pb_reset_category_colors.setText(QCoreApplication.translate("prefDialog", u"Reset colors to default", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_colors), QCoreApplication.translate("prefDialog", u"Plot colors", None))
        self.label_9.setText(QCoreApplication.translate("prefDialog", u"Toolbar icons size", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_interface), QCoreApplication.translate("prefDialog", u"Interface", None))
        self.pb_refresh.setText(QCoreApplication.translate("prefDialog", u"Refresh", None))
        self.pbCancel.setText(QCoreApplication.translate("prefDialog", u"Cancel", None))
        self.pbOK.setText(QCoreApplication.translate("prefDialog", u"OK", None))
    # retranslateUi




from .const import Bound
from .interval import Interval, singleton

from collections.abc import MutableMapping, Mapping

from sortedcontainers import SortedDict


def _sort(i):
    # Sort by lower bound, closed first
    return (i[0].lower, i[0].left is Bound.OPEN)


class IntervalDict(MutableMapping):
    """
    An IntervalDict is a dict-like data structure that maps from intervals to data,
    where keys can be single values or Interval instances.

    When keys are Interval instances, its behaviour merely corresponds to
    range queries and it returns IntervalDict instances corresponding to the
    subset of values covered by the given interval. If no matching value is
    found, an empty IntervalDict is returned.
    When keys are "single values", its behaviour corresponds to the one of Python
    built-in dict. When no matchin value is found, a KeyError is raised.

    Note that this class does not aim to have the best performance, but is
    provided mainly for convenience. Its performance mainly depends on the
    number of distinct values (not keys) that are stored.
    """

    __slots__ = ('_storage', )

    def __init__(self, mapping_or_iterable=None):
        """
        Return a new IntervalDict.

        If no argument is given, an empty IntervalDict is created. If an argument
        is given, and is a mapping object (e.g., another IntervalDict), an
        new IntervalDict with the same key-value pairs is created. If an
        iterable is provided, it has to be a list of (key, value) pairs.

        :param mapping_or_iterable: optional mapping or iterable.
        """
        self._storage = SortedDict(_sort)  # Mapping from intervals to values

        if mapping_or_iterable is not None:
            self.update(mapping_or_iterable)

    @classmethod
    def _from_items(cls, items):
        """
        Fast creation of an IntervalDict with the provided items.

        The items have to satisfy the two following properties: (1) all keys
        must be disjoint intervals and (2) all values must be distinct.

        :param items: list of (key, value) pairs.
        :return: an IntervalDict
        """
        d = cls()
        for key, value in items:
            d._storage[key] = value

        return d

    def clear(self):
        """
        Remove all items from the IntervalDict.
        """
        self._storage.clear()

    def copy(self):
        """
        Return a shallow copy.

        :return: a shallow copy.
        """
        return IntervalDict._from_items(self.items())

    def get(self, key, default=None):
        """
        Return the values associated to given key.

        If the key is a single value, it returns a single value (if it exists) or
        the default value. If the key is an Interval, it returns a new IntervalDict
        restricted to given interval. In that case, the default value is used to
        "fill the gaps" (if any) w.r.t. given key.

        :param key: a single value or an Interval instance.
        :param default: default value (default to None).
        :return: an IntervalDict, or a single value if key is not an Interval.
        """
        if isinstance(key, Interval):
            d = self[key]
            d[key - d.domain()] = default
            return d
        else:
            try:
                return self[key]
            except KeyError:
                return default

    def find(self, value):
        """
        Return a (possibly empty) Interval i such that self[i] = value, and
        self[~i] != value.

        :param value: value to look for.
        :return: an Interval instance.
        """
        return Interval(*(i for i, v in self._storage.items() if v == value))

    def items(self):
        """
        Return a view object on the contained items sorted by key
        (see https://docs.python.org/3/library/stdtypes.html#dict-views).

        :return: a view object.
        """
        return self._storage.items()

    def keys(self):
        """
        Return a view object on the contained keys (sorted)
        (see https://docs.python.org/3/library/stdtypes.html#dict-views).

        :return: a view object.
        """
        return self._storage.keys()

    def values(self):
        """
        Return a view object on the contained values sorted by key
        (see https://docs.python.org/3/library/stdtypes.html#dict-views).

        :return: a view object.
        """
        return self._storage.values()

    def domain(self):
        """
        Return an Interval corresponding to the domain of this IntervalDict.

        :return: an Interval.
        """
        return Interval(*self._storage.keys())

    def pop(self, key, default=None):
        """
        Remove key and return the corresponding value if key is not an Interval.
        If key is an interval, it returns an IntervalDict instance.

        This method combines self[key] and del self[key]. If a default value
        is provided and is not None, it uses self.get(key, default) instead of
        self[key].

        :param key: a single value or an Interval instance.
        :param default: optional default value.
        :return: an IntervalDict, or a single value if key is not an Interval.
        """
        if default is None:
            value = self[key]
            del self[key]
            return value
        else:
            value = self.get(key, default)
            try:
                del self[key]
            except KeyError:
                pass
            return value

    def popitem(self):
        """
        Remove and return some (key, value) pair as a 2-tuple.
        Raise KeyError if D is empty.

        :return: a (key, value) pair.
        """
        return self._storage.popitem()

    def setdefault(self, key, default=None):
        """
        Return given key. If it does not exist, set its value to default and
        return it.

        :param key: a single value or an Interval instance.
        :param default: default value (default to None).
        :return: an IntervalDict, or a single value if key is not an Interval.
        """
        if isinstance(key, Interval):
            value = self.get(key, default)
            self.update(value)
            return value
        else:
            try:
                return self[key]
            except KeyError:
                self[key] = default
                return default

    def update(self, mapping_or_iterable):
        """
        Update current IntervalDict with provided values.

        If a mapping is provided, it must map Interval instances to values (e.g., another
        IntervalDict). If an iterable is provided, it must consist of a list of
        (key, value) pairs.

        :param mapping_or_iterable: mapping or iterable.
        """
        if isinstance(mapping_or_iterable, Mapping):
            data = mapping_or_iterable.items()
        else:
            data = mapping_or_iterable

        for i, v in data:
            self[i] = v

    def combine(self, other, how):
        """
        Return a new IntervalDict that combines the values from current and
        provided ones.

        If d = d1.combine(d2, f), then d contains (1) all values from d1 whose
        keys do not intersect the ones of d2, (2) all values from d2 whose keys
        do not intersect the ones of d1, and (3) f(x, y) for x in d1, y in d2 for
        intersecting keys.

        :param other: another IntervalDict instance.
        :param how: a function of two parameters that combines values.
        :return: a new IntervalDict instance.
        """
        new_items = []

        dom1, dom2 = self.domain(), other.domain()

        new_items.extend(self[dom1 - dom2].items())
        new_items.extend(other[dom2 - dom1].items())

        intersection = dom1 & dom2
        d1, d2 = self[intersection], other[intersection]

        for i1, v1 in d1.items():
            for i2, v2 in d2.items():
                if i1.overlaps(i2):
                    i = i1 & i2
                    v = how(v1, v2)
                    new_items.append((i, v))

        return IntervalDict(new_items)

    def as_dict(self):
        """
        Return the content as a classical Python dict.

        :return: a Python dict.
        """
        return dict(self._storage)

    def __getitem__(self, key):
        if isinstance(key, Interval):
            items = []
            for i, v in self._storage.items():
                intersection = key & i
                if not intersection.empty:
                    items.append((intersection, v))
            return IntervalDict._from_items(items)
        else:
            for i, v in self._storage.items():
                if key in i:
                    return v
            raise KeyError(key)

    def __setitem__(self, key, value):
        interval = key if isinstance(key, Interval) else singleton(key)

        if interval.empty:
            return

        removed_keys = []
        added_items = []

        found = False
        for i, v in self._storage.items():
            if value == v:
                found = True
                # Extend existing key
                removed_keys.append(i)
                added_items.append((i | interval, v))
            elif i.overlaps(interval):
                # Reduce existing key
                remaining = i - interval
                removed_keys.append(i)
                if not remaining.empty:
                    added_items.append((remaining, v))

        if not found:
            added_items.append((interval, value))

        # Update storage accordingly
        for key in removed_keys:
            self._storage.pop(key)

        for key, value in added_items:
            self._storage[key] = value

    def __delitem__(self, key):
        interval = key if isinstance(key, Interval) else singleton(key)

        if interval.empty:
            return

        removed_keys = []
        added_items = []

        found = False
        for i, v in self._storage.items():
            if i.overlaps(interval):
                found = True
                remaining = i - interval
                removed_keys.append(i)
                if not remaining.empty:
                    added_items.append((remaining, v))

        if not found and not isinstance(key, Interval):
            raise KeyError(key)

        # Update storage accordingly
        for key in removed_keys:
            self._storage.pop(key)

        for key, value in added_items:
            self._storage[key] = value

    def __or__(self, other):
        d = self.copy()
        d.update(other)
        return d

    def __ior__(self, other):
        self.update(other)
        return self

    def __iter__(self):
        return iter(self._storage)

    def __len__(self):
        return len(self._storage)

    def __contains__(self, key):
        return key in self.domain()

    def __repr__(self):
        return '{}{}{}'.format(
            '{',
            ', '.join('{!r}: {!r}'.format(i, v) for i, v in self.items()),
            '}',
        )

    def __eq__(self, other):
        if isinstance(other, IntervalDict):
            return self.as_dict() == other.as_dict()
        else:
            return NotImplemented



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2020 Olivier Friard

module for testing utilities.py

https://realpython.com/python-continuous-integration/

pytest test_utilities.py
"""

import pytest
import os
import sys
import shutil
import subprocess
from decimal import Decimal
import json
import datetime
import numpy as np
from pathlib import Path

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from boris import utilities
from boris import config


@pytest.fixture()
def before():
    if Path("output").is_dir():
        shutil.rmtree("output")
    os.makedirs("output", exist_ok=True)


class Test_accurate_media_analysis(object):
    def test_ffmpeg_available(self):
        result = subprocess.run(["ffmpeg", "-version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
        assert "ffmpeg version" in result.stdout.decode()

    def test_media_ok(self):
        r = utilities.accurate_media_analysis("ffmpeg", "files/geese1.mp4")

        assert r == {
            "analysis_program": "ffprobe",
            "frames_number": 1548,
            "duration_ms": 61920.0,
            "duration": 61.92,
            "audio_duration": 61.92,
            "fps": 25.0,
            "has_video": True,
            "has_audio": True,
            "bitrate": 642009,
            "resolution": "640x480",
            "sample_rate": 44100.0,
            "file size": 6977208,
            "audio_codec": "AAC (Advanced Audio Coding)",
            "video_codec": "H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10",
            "creation_time": "2014-06-24 19:52:17",
            "format_long_name": "QuickTime / MOV",
        }

    def test_no_media(self):
        r = utilities.accurate_media_analysis("ffmpeg", "files/test.boris")
        assert "error" in r

    def test_media_does_not_exist(self):
        r = utilities.accurate_media_analysis("ffmpeg", "files/xxx")
        assert "error" in r


class Test_angle(object):
    def test_1(self):
        round(utilities.angle((0, 0), (0, 90), (90, 0)), 3) == 90.0

    def test_2(self):
        assert round(utilities.angle((0, 0), (90, 0), (90, 0)), 3) == 0.0

    def test_3(self):
        assert round(utilities.angle((0, 0), (90, 0), (0, 90)), 3) == 90.0


class Test_oriented_angle(object):
    def test_angle0(self):
        # angle must be 0
        angle = utilities.oriented_angle((0, 0), (1, 0), (1, 0))
        assert angle == 0.0

    def test_angle_minus90(self):
        # angle must be 0
        angle = utilities.oriented_angle((0, 0), (1, 0), (0, 1))
        # print(angle)
        assert angle == -90.0

    def test_angle_90(self):
        # angle must be 90
        angle = utilities.oriented_angle((0, 0), (1, 0), (0, -1))
        assert angle == 90.0

    def test_angle_180(self):
        # angle must be -180
        angle = utilities.oriented_angle((0, 0), (1, 0), (-1, 0))
        print(angle)
        assert angle == -180.0


class Test_oriented_angle_trigo(object):
    def test_orientated_angle_trigo_0(self):
        # angle must be 0
        angle = utilities.oriented_angle_trigo((0, 0), (1, 0), (1, 0))
        assert angle == 0.0

    def test_orientated_angle_trigo_90(self):
        # angle must be 90
        angle = utilities.oriented_angle_trigo((0, 0), (1, 0), (0, 1))
        # print(angle)
        assert angle == 90.0

    def test_orientated_angle_trigo_minus90(self):
        # angle must be 270
        angle = utilities.oriented_angle_trigo((0, 0), (1, 0), (0, -1))
        assert angle == 270

    def test_orientated_angle_trigo_180(self):
        # angle must be 180
        angle = utilities.oriented_angle_trigo((0, 0), (1, 0), (-1, 0))
        assert angle == 180.0


"""
class Test_behavior_color(object):
    def test_idx0(self):
        assert utilities.behavior_color(config.BEHAVIORS_PLOT_COLORS, 0) == "tab:blue"

    def test_idx1000(self):
        assert utilities.behavior_color(config.BEHAVIORS_PLOT_COLORS, 1000) == "midnightblue"



class Test_check_txt_file(object):
    def test_csv(self):
        r = utilities.check_txt_file("files/test_check_txt_file_test_csv.csv")
        assert r == {"homogeneous": True, "fields number": 7, "separator": ","}

    def test_tsv(self):
        r = utilities.check_txt_file("files/test_check_txt_file_test_tsv.tsv")
        assert r == {"homogeneous": True, "fields number": 5, "separator": "\t"}

    def test_no_homogeneous(self):
        r = utilities.check_txt_file("files/test.boris")
        assert r == {"homogeneous": False}

    def test_file_does_not_exists(self):
        r = utilities.check_txt_file("files/xxx")
        assert r == {"error": "[Errno 2] No such file or directory: 'files/xxx'"}

    def test_empty_file(self):
        r = utilities.check_txt_file("files/empty_file")
        assert r == {"error": "Could not determine delimiter"}

"""


class Test_complete(object):
    def test_list_3_to_8(self):
        assert utilities.complete(["a", "b", "c"], 8) == ["a", "b", "c", "", "", "", "", ""]

    def test_empty_list(self):
        assert utilities.complete([], 4) == ["", "", "", ""]

    def test_list_longer_then_max(self):
        assert utilities.complete(["a", "b", "c", "d"], 3) == ["a", "b", "c", "d"]


class Test_convert_time_to_decimal(object):
    def test_1(self):
        pj = json.loads(open("files/test2.boris").read())
        r = utilities.convert_time_to_decimal(pj)

        txt = open("files/test.txt").read()
        pj_dec = eval(txt)
        assert r == pj_dec


class Test_datetime_iso8601(object):
    def test_1(self):
        r = utilities.datetime_iso8601(datetime.datetime(2018, 12, 1, 22, 36, 7, 652523))
        assert r == "2018-12-01 22:36:07"


"""

class Test_decimal_default(object):
    def test_1(self):
        assert utilities.decimal_default(Decimal("1.456")) == 1.456
        assert isinstance(utilities.decimal_default(Decimal("1.456")), float)
"""


class Test_distance(object):
    def test_1(self):
        utilities.distance((10, 10), (80, 120)) == 130.38404810405297

    def test_0(self):
        utilities.distance((10, 10), (10, 10)) == 0.0


class Test_eol2space(object):
    def test_rn(self):
        assert utilities.eol2space("aaa\r\nbbb") == "aaa bbb"

    def test_n(self):
        assert utilities.eol2space("aaa\nbbb") == "aaa bbb"

    def test_r(self):
        assert utilities.eol2space("aaa\rbbb") == "aaa bbb"


class Test_error_info(object):
    def test_error1(self):
        try:
            1 / 0
        except Exception:
            r = utilities.error_info(sys.exc_info())
            assert str(r[0]) == "<class 'ZeroDivisionError'>: division by zero"
            assert r[1] == "test_utilities.py"


class Test_extract_wav(object):
    @pytest.mark.usefixtures("before")
    def test_wav_from_mp4(self):
        r = utilities.extract_wav(ffmpeg_bin="ffmpeg", media_file_path="files/geese1.mp4", tmp_dir="output")
        print(r)
        assert Path(r) == Path("output") / Path("geese1.mp4.wav")
        assert (Path("output") / Path("geese1.mp4.wav")).is_file()


class Test_float2decimal(object):
    def test_1(self):
        r = utilities.float2decimal(0.001)
        assert r == Decimal(str(0.001))


class Test_get_current_points_by_subject(object):
    def test_no_events(self):
        pj = json.loads(open("files/test.boris").read())
        r = utilities.get_current_points_by_subject(
            point_behaviors_codes=["p"],
            events=pj[config.OBSERVATIONS]["observation #1"]["events"],
            subjects=pj["subjects_conf"],
            time=Decimal("3"),
            tolerance=Decimal("3"),
        )
        assert r == {"0": [], "1": []}

    def test_events_with_modifiers1(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_points_by_subject(
            point_behaviors_codes=["p"],
            events=pj[config.OBSERVATIONS]["offset positif"]["events"],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time=Decimal("22.000"),
            tolerance=Decimal("1"),
            include_modifiers=True,
        )
        assert r == {"0": [("p", "")]}

    def test_events_with_modifiers2(self):
        # no events should correspond to selected behavior
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_points_by_subject(
            point_behaviors_codes=["p"],
            events=pj[config.OBSERVATIONS]["modifiers"][config.EVENTS],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time=Decimal("8.000"),
            tolerance=Decimal("5"),
            include_modifiers=True,
        )
        assert r == {"0": []}

    def test_events_with_modifiers3(self):
        # no events should correspond to selected behavior
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_points_by_subject(
            point_behaviors_codes=["q"],
            events=pj[config.OBSERVATIONS]["modifiers"][config.EVENTS],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time=Decimal("8.000"),
            tolerance=Decimal("5"),
            include_modifiers=True,
        )
        assert r == {"0": [("q", "m1"), ("q", "m2")]}

    def test_events_without_modifiers1(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_points_by_subject(
            point_behaviors_codes=["p"],
            events=pj[config.OBSERVATIONS]["offset positif"]["events"],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time=Decimal("22.000"),
            tolerance=Decimal("1"),
            include_modifiers=False,
        )
        assert r == {"0": [("p", "")]}

    def test_events_without_modifiers2(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_points_by_subject(
            point_behaviors_codes=["p"],
            events=pj[config.OBSERVATIONS]["modifiers"]["events"],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time=Decimal("8.000"),
            tolerance=Decimal("5"),
            include_modifiers=True,
        )
        assert r == {"0": []}


class Test_get_current_states_by_subject(object):
    def test_t0(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["s"],
            events=pj["observations"]["observation #1"]["events"],
            subjects=pj["subjects_conf"],
            time_=Decimal("0"),
            include_modifiers=False,
        )
        assert r == {"0": [], "1": []}

    def test_t4(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["s"],
            events=pj["observations"]["observation #1"]["events"],
            subjects=pj["subjects_conf"],
            time_=Decimal("4.0"),
            include_modifiers=False,
        )
        assert r == {"0": ["s"], "1": []}

    def test_t8(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["s"],
            events=pj["observations"]["observation #1"]["events"],
            subjects=pj["subjects_conf"],
            time_=Decimal("8.0"),
            include_modifiers=False,
        )
        assert r == {"0": [], "1": []}

    def test_t_neg(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["s"],
            events=pj["observations"]["observation #1"]["events"],
            subjects=pj["subjects_conf"],
            time_=Decimal("-12.456"),
            include_modifiers=False,
        )
        # print(r)
        assert r == {"0": [], "1": []}

    def test_no_state_events(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["s"],
            events=pj["observations"]["offset positif"]["events"],
            subjects=pj["subjects_conf"],
            time_=Decimal("30"),
            include_modifiers=False,
        )
        # print(r)
        assert r == {"0": [], "1": []}

    def test_events_with_modifiers_not_required(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["r", "s"],
            events=pj["observations"]["modifiers"]["events"],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time_=Decimal("20"),
            include_modifiers=False,
        )
        # print(r)
        assert r == {"0": ["r"]}

    def test_events_with_modifiers_required(self):
        pj_float = json.loads(open("files/test.boris").read())
        pj = utilities.convert_time_to_decimal(pj_float)
        r = utilities.get_current_states_modifiers_by_subject(
            state_behaviors_codes=["r", "s"],
            events=pj["observations"]["modifiers"]["events"],
            subjects={"0": {"key": "", "name": "", "description": "no focal subject"}},
            time_=Decimal("20"),
            include_modifiers=True,
        )
        # print(r)
        assert r == {"0": ["r (m1)"]}


class Test_intfloatstr(object):
    def test_str(self):
        assert utilities.intfloatstr("abc") == "abc"

    def test_int(self):
        assert utilities.intfloatstr("8") == 8

    def test_float(self):
        assert utilities.intfloatstr("1.458") == "1.458"


class Test_polygon_area(object):
    def test_polygon(self):
        assert round(utilities.polygon_area([(0, 0), (90, 0), (0, 90)])) == 4050

    def test_empty_polygon(self):
        assert round(utilities.polygon_area([(0, 0), (90, 0), (0, 90), (90, 90)])) == 0


class Test_return_file_header(object):
    def test_file_ok(self):
        r = utilities.return_file_header("files/test_export_events_tabular.tsv")
        # print(r)
        assert r == [
            "Observation id\tobservation #1\t\t\t\t\t\t\t\n",
            "\t\t\t\t\t\t\t\t\n",
            "Media file(s)\t\t\t\t\t\t\t\t\n",
            "\t\t\t\t\t\t\t\t\n",
            "Player #1\tvideo_test_25fps_360s.mp4\t\t\t\t\t\t\t\n",
        ]

    def test_no_file(self):
        r = utilities.return_file_header("files/xxx")
        assert r == []

    def test_short_file(self):
        r = utilities.return_file_header("files/Test_events_to_behavioral_sequences_test_5_observation_not_paired")
        assert r == ["p|s|s+p|s+p|p|s|s+p|s|s", "", "", "", ""]


class Test_safefilename(object):
    def test_filename_with_spaces(self):
        assert utilities.safeFileName("aaa bbb.ccc") == "aaa bbb.ccc"

    def test_filename_with_forbidden_chars(self):
        # ["/", "\\", ":", "*", "?", '"', "<", ">", "|"]
        assert utilities.safeFileName('aaa/bb\\b.c:cc ddd* ? "www" <> |') == "aaa_bb_b.c_cc ddd_ _ _www_ __ _"


class Test_safe_xl_worksheet_title(object):
    def test_long_title_xls(self):
        assert utilities.safe_xl_worksheet_title("0123456789012345678901234567890123456789", "xls") == "0123456789012345678901234567890"

    def test_long_title_xlsx(self):
        assert (
            utilities.safe_xl_worksheet_title("0123456789012345678901234567890123456789", "xlsx")
            == "0123456789012345678901234567890123456789"
        )

    def test_long_title_tsv(self):
        assert (
            utilities.safe_xl_worksheet_title("0123456789012345678901234567890123456789", "tsv")
            == "0123456789012345678901234567890123456789"
        )

    def test_title_with_forbidden_chars_xls(self):
        assert utilities.safe_xl_worksheet_title(r"000\000/000*000[000]000:000?000 000", "xls") == "000 000 000 000 000 000 000 000"

    def test_title_with_forbidden_chars_xlsx(self):
        assert utilities.safe_xl_worksheet_title(r"000\000/000*000[000]000:000?000 000", "xls") == "000 000 000 000 000 000 000 000"


class Test_seconds_of_day(object):
    def test1(self):
        assert utilities.seconds_of_day(datetime.datetime(2002, 12, 25, 0, 0, 10, 123)) == Decimal("10.000")

    def test2(self):
        assert utilities.seconds_of_day(datetime.datetime(2002, 12, 25, 0, 0, 10, 123000)) == Decimal("10.123")

    def test3(self):
        assert utilities.seconds_of_day(datetime.datetime(2002, 12, 25, 1, 2, 3, 654321)) == Decimal("3723.654")


class Test_seconds2time(object):
    def test_negative(self):
        assert utilities.seconds2time(Decimal(-2.123)) == "-00:00:02.123"

    def test_gt_86400(self):
        assert utilities.seconds2time(Decimal(86400.999)) == "24:00:00.999"

    def test_10(self):
        assert utilities.seconds2time(Decimal(10.0)) == "00:00:10.000"


class Test_sorted_keys(object):
    def test_numeric_keys(self):
        r = utilities.sorted_keys({5: "a", 4: "7", 0: "z", 6: "a"})
        assert r == ["0", "4", "5", "6"]

    def test_str_keys(self):
        r = utilities.sorted_keys({"10": "x", "0": "x", "1": "x", "11": "x", "05": "x"})
        assert r == ["0", "1", "5", "10", "11"]

    def test_empty_dict(self):
        r = utilities.sorted_keys({})
        assert r == []


class Test_state_behavior_codes(object):
    def test_1(self):
        pj_float = json.loads(open("files/test.boris").read())
        r = utilities.state_behavior_codes(pj_float["behaviors_conf"])
        # print(r)
        assert r == ["s", "r", "m"]

    def test_empty_ethogram(self):
        r = utilities.state_behavior_codes({})
        assert r == []


class Test_test_ffmpeg_path(object):
    def test_path_ok(self):
        r = utilities.test_ffmpeg_path("ffmpeg")
        # print(r)
        assert r == (True, "")

    def test_path_do_not_exist(self):
        r = utilities.test_ffmpeg_path("xxx/ffmpeg")
        print(r)
        assert r == (False, "FFmpeg is required but it was not found.<br>See https://www.ffmpeg.org")


class Test_time2seconds(object):
    def test_positive(self):
        assert utilities.time2seconds("11:22:33.44") == Decimal("40953.44")

    def test_negative(self):
        assert utilities.time2seconds("-11:22:33.44") == Decimal("-40953.44")

    def test_zero(self):
        assert utilities.time2seconds("00:00:00.000") == Decimal("0.000")

    def test_wrong_input(self):
        assert utilities.time2seconds("aaaaa") == Decimal("0.000")


class Test_txt2np_array(object):
    def test_no_file(self):
        r = utilities.txt2np_array(
            file_name="files/xxx", columns_str="4,6", substract_first_value="False", converters={}, column_converter={}
        )
        # print(r)
        assert r[0] is False
        assert r[1] == "[Errno 2] No such file or directory: 'files/xxx'"
        assert list(r[2].shape) == [0]

    def test_file_csv_no_converter(self):
        r = utilities.txt2np_array(
            file_name="files/test_check_txt_file_test_csv.csv",
            columns_str="4,6",
            substract_first_value="False",
            converters={},
            column_converter={},
        )

        print(r)

        assert r[0] is False
        assert r[1] == "could not convert string to float: '14:38:58'"
        assert list(r[2].shape) == [0]

    def test_file_csv_converter(self):
        r = utilities.txt2np_array(
            file_name="files/test_check_txt_file_test_csv.csv",
            columns_str="4,6",
            substract_first_value="False",
            converters={
                "HHMMSS_2_seconds": {
                    "name": "HHMMSS_2_seconds",
                    "description": "convert HH:MM:SS in seconds since 1970-01-01",
                    "code": "\nh, m, s = INPUT.split(':')\nOUTPUT = int(h) * 3600 + int(m) * 60 + int(s)\n\n",
                }
            },
            column_converter={4: "HHMMSS_2_seconds"},
        )
        assert r[0] is True
        assert r[1] == ""
        assert r[2][0, 0] == 52738.0
        assert r[2][1, 0] == 52740.0
        assert r[2][0, 1] == 12.4144278
        assert list(r[2].shape) == [10658, 2]


"""

class Test_url2path(object):
    def test_1(self):
        assert utilities.url2path("file:///home/olivier/v%C3%A9lo/test") == "/home/olivier/vélo/test"


class Test_versiontuple(object):
    def test_1(self):
        r = utilities.versiontuple("1.2.3")
        assert r == ("00000001", "00000002", "00000003")

    def test_2(self):
        r = utilities.versiontuple("1.2")
        assert r == ("00000001", "00000002")

    def test_3(self):
        r = utilities.versiontuple("1")
        assert r == ("00000001",)

    def test_4(self):
        r = utilities.versiontuple("")
        assert r == ("00000000",)
"""



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2025 Olivier Friard


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.

"""

from optparse import OptionParser


def parse_arguments():
    # check if argument
    usage = 'usage: %prog [options] [-p PROJECT_PATH] [-o "OBSERVATION ID"]'
    parser = OptionParser(usage=usage)

    parser.add_option("-d", "--debug", action="store_true", default=False, dest="debug", help="Use debugging mode")
    parser.add_option("-v", "--version", action="store_true", default=False, dest="version", help="Print version")
    parser.add_option("-n", "--nosplashscreen", action="store_true", default=False, help="No splash screen")
    parser.add_option("-p", "--project", action="store", default="", dest="project", help="Project file")
    parser.add_option("-o", "--observation", action="store", default="", dest="observation", help="Observation id")
    parser.add_option(
        "-f",
        "--no-first-launch-dialog",
        action="store_true",
        default=False,
        dest="no_first_launch_dialog",
        help="No first launch dialog (for new version automatic check)",
    )

    return parser.parse_args()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'add_modifier.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QDialog,
    QHBoxLayout, QLabel, QLineEdit, QListWidget,
    QListWidgetItem, QPushButton, QSizePolicy, QSpacerItem,
    QTabWidget, QVBoxLayout, QWidget)

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(1088, 654)
        self.verticalLayout_5 = QVBoxLayout(Dialog)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.cb_ask_at_stop = QCheckBox(Dialog)
        self.cb_ask_at_stop.setObjectName(u"cb_ask_at_stop")

        self.verticalLayout_5.addWidget(self.cb_ask_at_stop)

        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.lbModifier = QLabel(Dialog)
        self.lbModifier.setObjectName(u"lbModifier")

        self.verticalLayout_2.addWidget(self.lbModifier)

        self.leModifier = QLineEdit(Dialog)
        self.leModifier.setObjectName(u"leModifier")

        self.verticalLayout_2.addWidget(self.leModifier)

        self.lbCode = QLabel(Dialog)
        self.lbCode.setObjectName(u"lbCode")

        self.verticalLayout_2.addWidget(self.lbCode)

        self.leCode = QLineEdit(Dialog)
        self.leCode.setObjectName(u"leCode")

        self.verticalLayout_2.addWidget(self.leCode)

        self.lbCodeHelp = QLabel(Dialog)
        self.lbCodeHelp.setObjectName(u"lbCodeHelp")
        self.lbCodeHelp.setWordWrap(True)

        self.verticalLayout_2.addWidget(self.lbCodeHelp)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)


        self.horizontalLayout_5.addLayout(self.verticalLayout_2)

        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.pbAddModifier = QPushButton(Dialog)
        self.pbAddModifier.setObjectName(u"pbAddModifier")

        self.verticalLayout_3.addWidget(self.pbAddModifier)

        self.pbModifyModifier = QPushButton(Dialog)
        self.pbModifyModifier.setObjectName(u"pbModifyModifier")

        self.verticalLayout_3.addWidget(self.pbModifyModifier)

        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_3.addItem(self.verticalSpacer_2)


        self.horizontalLayout_5.addLayout(self.verticalLayout_3)

        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.tabWidgetModifiersSets = QTabWidget(Dialog)
        self.tabWidgetModifiersSets.setObjectName(u"tabWidgetModifiersSets")
        self.tabWidgetModifiersSets.setMaximumSize(QSize(16777215, 30))
        self.tabWidgetModifiersSets.setTabPosition(QTabWidget.TabPosition.North)
        self.tabWidgetModifiersSets.setTabShape(QTabWidget.TabShape.Rounded)
        self.tabWidgetModifiersSets.setElideMode(Qt.TextElideMode.ElideNone)
        self.tabWidgetModifiersSets.setDocumentMode(True)

        self.verticalLayout.addWidget(self.tabWidgetModifiersSets)

        self.lb_name = QLabel(Dialog)
        self.lb_name.setObjectName(u"lb_name")

        self.verticalLayout.addWidget(self.lb_name)

        self.le_name = QLineEdit(Dialog)
        self.le_name.setObjectName(u"le_name")

        self.verticalLayout.addWidget(self.le_name)

        self.lb_description = QLabel(Dialog)
        self.lb_description.setObjectName(u"lb_description")

        self.verticalLayout.addWidget(self.lb_description)

        self.le_description = QLineEdit(Dialog)
        self.le_description.setObjectName(u"le_description")

        self.verticalLayout.addWidget(self.le_description)

        self.lbType = QLabel(Dialog)
        self.lbType.setObjectName(u"lbType")

        self.verticalLayout.addWidget(self.lbType)

        self.cbType = QComboBox(Dialog)
        self.cbType.addItem("")
        self.cbType.addItem("")
        self.cbType.addItem("")
        self.cbType.addItem("")
        self.cbType.setObjectName(u"cbType")

        self.verticalLayout.addWidget(self.cbType)

        self.lbValues = QLabel(Dialog)
        self.lbValues.setObjectName(u"lbValues")

        self.verticalLayout.addWidget(self.lbValues)

        self.lwModifiers = QListWidget(Dialog)
        self.lwModifiers.setObjectName(u"lwModifiers")

        self.verticalLayout.addWidget(self.lwModifiers)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.pbMoveUp = QPushButton(Dialog)
        self.pbMoveUp.setObjectName(u"pbMoveUp")

        self.horizontalLayout.addWidget(self.pbMoveUp)

        self.pbMoveDown = QPushButton(Dialog)
        self.pbMoveDown.setObjectName(u"pbMoveDown")

        self.horizontalLayout.addWidget(self.pbMoveDown)

        self.pb_sort_modifiers = QPushButton(Dialog)
        self.pb_sort_modifiers.setObjectName(u"pb_sort_modifiers")

        self.horizontalLayout.addWidget(self.pb_sort_modifiers)

        self.pbRemoveModifier = QPushButton(Dialog)
        self.pbRemoveModifier.setObjectName(u"pbRemoveModifier")

        self.horizontalLayout.addWidget(self.pbRemoveModifier)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.pbAddSet = QPushButton(Dialog)
        self.pbAddSet.setObjectName(u"pbAddSet")

        self.horizontalLayout_3.addWidget(self.pbAddSet)

        self.pbMoveSetLeft = QPushButton(Dialog)
        self.pbMoveSetLeft.setObjectName(u"pbMoveSetLeft")

        self.horizontalLayout_3.addWidget(self.pbMoveSetLeft)

        self.pbMoveSetRight = QPushButton(Dialog)
        self.pbMoveSetRight.setObjectName(u"pbMoveSetRight")

        self.horizontalLayout_3.addWidget(self.pbMoveSetRight)

        self.pbRemoveSet = QPushButton(Dialog)
        self.pbRemoveSet.setObjectName(u"pbRemoveSet")

        self.horizontalLayout_3.addWidget(self.pbRemoveSet)


        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")

        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.pb_add_subjects = QPushButton(Dialog)
        self.pb_add_subjects.setObjectName(u"pb_add_subjects")

        self.verticalLayout.addWidget(self.pb_add_subjects)

        self.pb_load_file = QPushButton(Dialog)
        self.pb_load_file.setObjectName(u"pb_load_file")

        self.verticalLayout.addWidget(self.pb_load_file)

        self.verticalSpacer_3 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer_3)


        self.horizontalLayout_5.addLayout(self.verticalLayout)


        self.verticalLayout_4.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.pbCancel = QPushButton(Dialog)
        self.pbCancel.setObjectName(u"pbCancel")

        self.horizontalLayout_2.addWidget(self.pbCancel)

        self.pbOK = QPushButton(Dialog)
        self.pbOK.setObjectName(u"pbOK")

        self.horizontalLayout_2.addWidget(self.pbOK)


        self.verticalLayout_4.addLayout(self.horizontalLayout_2)


        self.verticalLayout_5.addLayout(self.verticalLayout_4)


        self.retranslateUi(Dialog)

        self.tabWidgetModifiersSets.setCurrentIndex(-1)


        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Set modifiers", None))
        self.cb_ask_at_stop.setText(QCoreApplication.translate("Dialog", u"Ask for modifier(s) when behavior stops", None))
        self.lbModifier.setText(QCoreApplication.translate("Dialog", u"Modifier", None))
        self.lbCode.setText(QCoreApplication.translate("Dialog", u"Key code", None))
        self.lbCodeHelp.setText(QCoreApplication.translate("Dialog", u"Key code is case sensitive. Type one character or a function key (F1, F2... F12)", None))
        self.pbAddModifier.setText(QCoreApplication.translate("Dialog", u"->", None))
        self.pbModifyModifier.setText(QCoreApplication.translate("Dialog", u"<-", None))
        self.lb_name.setText(QCoreApplication.translate("Dialog", u"Set name", None))
        self.lb_description.setText(QCoreApplication.translate("Dialog", u"Description", None))
        self.lbType.setText(QCoreApplication.translate("Dialog", u"Modifier type", None))
        self.cbType.setItemText(0, QCoreApplication.translate("Dialog", u"Single selection", None))
        self.cbType.setItemText(1, QCoreApplication.translate("Dialog", u"Multiple selection", None))
        self.cbType.setItemText(2, QCoreApplication.translate("Dialog", u"Numeric", None))
        self.cbType.setItemText(3, QCoreApplication.translate("Dialog", u"Value from external data file", None))

        self.lbValues.setText(QCoreApplication.translate("Dialog", u"Values", None))
        self.pbMoveUp.setText(QCoreApplication.translate("Dialog", u"Move modifier up", None))
        self.pbMoveDown.setText(QCoreApplication.translate("Dialog", u"Move modifier down", None))
        self.pb_sort_modifiers.setText(QCoreApplication.translate("Dialog", u"Sort modifiers", None))
        self.pbRemoveModifier.setText(QCoreApplication.translate("Dialog", u"Remove modifier", None))
        self.pbAddSet.setText(QCoreApplication.translate("Dialog", u"Add set of modifiers", None))
        self.pbMoveSetLeft.setText(QCoreApplication.translate("Dialog", u"Move set left", None))
        self.pbMoveSetRight.setText(QCoreApplication.translate("Dialog", u"Move set right", None))
        self.pbRemoveSet.setText(QCoreApplication.translate("Dialog", u"Remove set of modifiers", None))
        self.pb_add_subjects.setText(QCoreApplication.translate("Dialog", u"Add subjects as modifiers", None))
        self.pb_load_file.setText(QCoreApplication.translate("Dialog", u"Load modifiers from file", None))
        self.pbCancel.setText(QCoreApplication.translate("Dialog", u"Cancel", None))
        self.pbOK.setText(QCoreApplication.translate("Dialog", u"OK", None))
    # retranslateUi

